#include "surfboc.h"
#include "globprep.h"
#include "globalt.h"
#include "intools.h"
#include "mathem.h"
#include <string.h>

surfboc::surfboc()
{
  lcid = 0;
  bct  = 0;
  pv = 0.0; 
  ccid = 0; 
  trc = 0.0;  
  trr = 0.0;
}

surfboc::~surfboc()
{
  delete tpv;
}



/**
  Function reads data for boundary condition on elements from the opened text 
  file given by parameter in.
  Parameters :
  @param in  - pointer to opened text file from which data will be read.
  @param nlc - number of load cases.
  Return :
  @retval  0 - on succes
  @retval  1 - error reading boundary condition type
  @retval  2 - unknown type of problem is required
  @retval  3 - unknown type of boundary condition
  @retval  4 - error reading prescribed value for Dirichlet condition
  @retval  5 - error reading flux value for Neumann condition
  @retval  6 - error reading climate condition index
  @retval  7 - error reading transmition
  @retval  8 - error reading special transmition
  @retval  9 - error reading load case id
  @retval 10 - error reading surface id
*/
long surfboc::read(FILE *in, long nlc)
{
  long ret = 0;
  
  if (getlong(in, lcid))  // reading load case id
  {
    fprintf(stderr, "\n\n Cannot read load case id in function bocone::read\n");
    fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
    return 9;
  }
  if ((lcid > nlc) || (lcid < 1))
  {
    fprintf(stderr, "\n\n Wrong load case id, it is %ld and it should be in <1,%ld>\n", lcid, nlc);
    fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
    return 9;
  }
  if (getlong(in, bct)) // reading type of boundary condition
  {
    fprintf(stderr, "\n\n Cannot read type of boundary condition in function bocone::read\n");
    fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
    return 1;
  }
  
  if (bct)
  {
    switch (Tp->tprob)
    {
      case stationary_problem :
        ret = read_statpr(in);
        break;
      case nonstationary_problem :
      case nonlinear_nonstationary_problem :
        ret = read_nonstatpr(in);
        break;
      default :
        ret = 2;
    }
  }

  fflush(stderr);
  switch(ret)
  {  
    case 0:
      break;
    case 2:
      fprintf(stderr, "\n\n Unknown type of problem required in function bocone::read\n");
      fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
      break;
    case 3:
      fprintf(stderr, "\n\n Unknown type of boundary condition is required\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
    case 4:
      fprintf(stderr, "\n\n Error reading prescribed value\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
    case 5:
      fprintf(stderr, "\n\n Error reading flux\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
    case 6:
      fprintf(stderr, "\n\n Error reading climate condition index\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
    case 7:
      fprintf(stderr, "\n\n Error reading transmition\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
    case 8:
      fprintf(stderr, "\n\n Error reading special transmition\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
    default :
      fprintf(stderr, "\n\n Error reading boundary condion on element\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
  }
  return ret;
}



/**
  Function reads boundary conditions on elements for stationary problem from the text 
  file given by the parameter in.
  Parameters :
  @param in - pointer to opened text file from which data will be read.
  Return :
  @retval 0 - on succes
  @retval 3 - unknown type of boundary condition
  @retval 4 - error reading prescribed value for Dirichlet condition
  @retval 5 - error reading flux value for Neumann condition
  @retval 6 - error reading climate condition index
  @retval 7 - error reading transmition
  @retval 8 - error reading special transmition
*/
long surfboc::read_statpr(FILE *in)
{
  switch(bct)
  {
    case 0:  // no condition
      break;
    case 1: // Dirichlet condition
      if (getdouble(in, pv))   return(4);  // reading prescribed values
      break;
    case 2: // Neumann condition
      if (getdouble(in, pv))   return(5);  // reading flux
      break;
    case 4: // climate condition
      if (getlong(in, ccid))   return(6);  // reading condition id
      break;
    case 30: // transmition
      if (getdouble(in, pv))   return(7);   // reading transmition
      if (getdouble(in, trc))  return(7);   // reading transmition coefficient
      break;
    case 90: // transmition special
      if (getdouble(in, pv))   return(8);   // reading transmition
      if (getdouble(in, trc))  return(8);   // reading transmition coefficient
      if (getdouble(in, trr))  return(8);   // reading another transmition coefficient
      break;
    default :
      return (3);
  }
  return 0;
}



/**
  Function reads boundary conditions on elements for nonstationary problem from the text 
  file given by the parameter in.
  Parameters :
  @param in - pointer to opened text file from which data will be read.
  Return :
  @retval 0 - on succes
  @retval 3 - unknown type of boundary condition
  @retval 4 - error reading prescribed value for Dirichlet condition
  @retval 5 - error reading flux value for Neumann condition
  @retval 6 - error reading climate condition index
  @retval 7 - error reading transmition
  @retval 8 - error reading special transmition
*/
long surfboc::read_nonstatpr(FILE *in)
{
  tpv = new gfunct;
  switch(bct)
  {
    case 0:  // no condition
      break;
    case 1: // Dirichlet condition
      if (tpv->read_prop(in))   return(4);  // reading prescribed values
      break;
    case 2: // Neumann condition
      if (tpv->read_prop(in))   return(5);  // reading flux
      break;
    case 4: // climate condition
      if (getlong(in, ccid))   return(6);  // reading condition id
      break;
    case 30: // transmition
      if (tpv->read_prop(in))  return(7);   // reading transmition
      if (getdouble(in, trc))  return(7);   // reading transmition coefficient
      break;
    case 90: // transmition special
      if (tpv->read_prop(in))  return(8);   // reading transmition
      if (getdouble(in, trc))  return(8);   // reading transmition coefficient
      if (getdouble(in, trr))  return(8);   // reading another transmition coefficient
      break;
    default :
      return (3);
  }
  return 0;
}
