/*
    File:             consol_awf2.cpp
    Author:           Tomas Krejci, 29/10/2009
    Purpose:          material model for air and water flow (saturated-nonsaturated) in a deforming porous medium
    sources:          Lewis and Schrefler pp. 93-97
    unknowns:         number of unknowns=2, pg,pw
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "constrel.h"
#include "consol_awf2.h"
#include "globalt.h"

con_awf2mat::con_awf2mat()
{
  //
  emod = 1.3e6;//Young's modulus [Pa]
  //
  nu = 0.4;//Poisson's ratio [-]
  //
  alpha = 1.0;//Biot's constant [-] alpha = 1 - kt/ks
  //
  ks = 1.0e12;//bulk modulus of solid phase (grains) [Pa]
  //
  kw = 2.0e9;//bulk modulus of water [Pa]
  //
  //ka = 0.1e6;//bulk modulus of air [Pa]
  //
  phi0 = 0.2975;//initial porosity [-]
  //
  kintr = 4.5e-13;//intrinsic permeability [m^2]
  //
  rhos = 2000.0;//solid grain density [kg/m^3]
  //
  rhow = 1000.0;//water density [kg/m^3]
  //
  muw = 1.0e-3;//water viscosity [Pa.s]
  //
  mug = 1.8e-5;//air viscosity [Pa.s]
  //
  patm = 101325.0;
  //
  temp = 293.15;
  //
}

con_awf2mat::~con_awf2mat()
{}


/**
   function reads parameters
   
   @param in - input file

   29/10/2009, TKr
*/
void con_awf2mat::read(XFILE *in)
{
  //xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", &emod, &nu, &rhos, &alpha, &ks, &phi0, &kw, &rhow, &muw, &kintr);
}


/**
   function reads parameters
   
   @param out - output file

   29/10/2009, TKr
*/
void con_awf2mat::print(FILE *out)
{
 
}




/**
   function computes conductivity matrix of the material
   in the required integration point
   
   @param d - conductivity %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

*/
void con_awf2mat::matcond (matrix &d,long ri,long ci,long ipp)
{
  long n;
  n = d.n;

  switch (n){
  case 1:{
    matcond1d (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    matcond3d (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}


/**
   function creates conductivity matrix of the material for 1D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2mat::matcond1d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,pg;
  k = 0.0;
  
  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = get_kww(pw,pg);
  if((ri == 0) && (ci == 1))
    k = get_kwg(pw,pg);
  if((ri == 1) && (ci == 0))
    k = get_kgw(pw,pg);
  if((ri == 1) && (ci == 1))
    k = get_kgg(pw,pg);
  
  d[0][0] = k;
}

/**
   function creates conductivity matrix of the material for 2D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2mat::matcond2d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,pg;
  k = 0.0;
  
  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = get_kww(pw,pg);
  if((ri == 0) && (ci == 1))
    k = get_kwg(pw,pg);
  if((ri == 1) && (ci == 0))
    k = get_kgw(pw,pg);
  if((ri == 1) && (ci == 1))
    k = get_kgg(pw,pg);
  
  fillm(0.0,d);

  d[0][0] = k;   d[0][1] = 0.0;
  d[1][0] = 0.0; d[1][1] = k;
}

/**
   function creates conductivity matrix of the material for 3D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/

void con_awf2mat::matcond3d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double pw,pg;
  k = 0.0;
  
  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    k = get_kww(pw,pg);
  if((ri == 0) && (ci == 1))
    k = get_kwg(pw,pg);
  if((ri == 1) && (ci == 0))
    k = get_kgw(pw,pg);
  if((ri == 1) && (ci == 1))
    k = get_kgg(pw,pg);
  
  fillm(0.0,d);
  
  d[0][0]=k;   d[0][1]=0.0; d[0][2]=0.0;
  d[1][0]=0.0; d[1][1]=k;   d[1][2]=0.0;
  d[2][0]=0.0; d[2][1]=0.0; d[2][2]=k;
}


/**
   function creates capacity %matrix of the material

   @param c   - capacity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf2mat::matcap (double &c,long ri,long ci,long ipp)
{
  double pw,pg;
  c = 0.0;
  
  pw = Tm->ip[ipp].av[0];
  pg = Tm->ip[ipp].av[1];
  
  if((ri == 0) && (ci == 0))
    c = get_capww(pw,pg);
  if((ri == 0) && (ci == 1))
    c = get_capwg(pw,pg);
  if((ri == 1) && (ci == 0))
    c = get_capgw(pw,pg);
  if((ri == 1) && (ci == 1))
    c = get_capgg(pw,pg);
}


/**
   function computes volume part of right-hand side matrix
   in the required integration point
   
   @param d - right-hand side %matrix of material
   ...
   @param ipp - number of integration point
   
   29/10/2009
*/
void con_awf2mat::rhs_volume (matrix &d,long ri,long ci,long ipp)
{
  long m;
  m = d.m;
  
  switch (m){
  case 1:{
    rhs1d1 (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    rhs2d1 (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    rhs3d1 (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}



/**
   function creates volume right-hand side matrix of the material for 1D problems
   @param d - right-hand %matrix of the material
   ...
   @param ipp - number of integration point
*/
void con_awf2mat::rhs1d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double g;
  
  pw = Tm->ip[ipp].av[0];// *scale_pc;//scaling
  pg = Tm->ip[ipp].av[1];// *scale_pg;//scaling
  
  if(ri == 0){
    f = get_fw1(pw,pg);
    g = Tp->gr1;//vector of gravity acceleration
    
    fillm(0.0,d);
    d[0][0] = f*g;
  }
  if(ri == 1){
    f = get_fg1(pw,pg);
    g = Tp->gr1;//vector of gravity acceleration

    fillm(0.0,d);
    d[0][0] = f*g;
  }
}

/**
   function creates volume right-hand side matrix of the material for 2D problems
   @param d - right-hand %matrix of the material
   ...
   @param ipp - number of integration point
*/
void con_awf2mat::rhs2d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double *g;
  
  pw = Tm->ip[ipp].av[0];// *scale_pc;//scaling
  pg = Tm->ip[ipp].av[1];// *scale_pg;//scaling
  
  if(ri == 0){
    g = new double [2];
    f = get_fw1(pw,pg);

    g[0] = Tp->gr1;//gravity acceleration
    g[1] = Tp->gr2;//gravity acceleration
    
    fillm(0.0,d);
    d[0][0] = f*g[0];
    d[1][0] = f*g[1];
  }
  if(ri == 1){
    g = new double [2];
    f = get_fg1(pw,pg);

    g[0] = Tp->gr1;//gravity acceleration
    g[1] = Tp->gr2;//gravity acceleration

    fillm(0.0,d);
    d[0][0] = f*g[0];
    d[1][0] = f*g[1];
  }
}

/**
   function creates volume right-hand side matrix of the material for 3D problems
   @param d - right-hand %matrix of the material
   ...
   @param ipp - number of integration point
*/
void con_awf2mat::rhs3d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double *g;
  
  pw = Tm->ip[ipp].av[0];// *scale_pc;//scaling
  pg = Tm->ip[ipp].av[1];// *scale_pg;//scaling
  
  if(ri == 0){
    g = new double [3];
    f = get_fw1(pw,pg);

    g[0] = Tp->gr1;//gravity acceleration
    g[1] = Tp->gr2;//gravity acceleration
    g[2] = Tp->gr3;//gravity acceleration

    fillm(0.0,d);
    d[0][0] = f*g[0];
    d[1][0] = f*g[1];
    d[2][0] = f*g[2];
  }
  if(ri == 1){
    g = new double [2];
    f = get_fg1(pw,pg);

    g[0] = Tp->gr1;//gravity acceleration
    g[1] = Tp->gr2;//gravity acceleration
    g[2] = Tp->gr3;//gravity acceleration

    fillm(0.0,d);
    d[0][0] = f*g[0];
    d[1][0] = f*g[1];
    d[2][0] = f*g[2];
  }
}


/**
   function computes degree of saturation(sorption curve)
   @param pw - water pressure
   @param pg - air pressure

   @retval sw - degree of saturation

   29/10/2009, TKr
*/
double con_awf2mat::get_sw(double pw, double pg)
{
  double sw,pc;
  //
  pc = pg - pw;
  
  sw = 1.0 - 1.9722e-11*pow(pc,2.4279);

  return(sw);
}



/**
   function computes specific moisture content
   @param pw - water pressure
   @param pg - air pressure

   @retval cs - specific moisture content

   29/10/2009, TKr
*/
double con_awf2mat::get_cs(double pw, double pg)
{
  double cs,pc;
  //
  pc = pg - pw;
  
  cs = -1.9722e-11*2.4279*pow(pc,(2.4279-1.0));

  return(cs);
}



/**
   function computes degree of saturation(sorption curve)
   @param sw - degree of saturation

   @retval se - degree of saturation

   29/10/2009, TKr
*/
double con_awf2mat::get_se(double sw)
{
  double se;
  //
  se = (sw - 0.2)*(1.0 - 0.2);

  return(se);
}




/**
   function computes water relative permeability
   @param sw - degree of saturraion
   
   @retval krl - water relative permeability

   12/9/2008, TKr
*/
double con_awf2mat::get_krl(double sw)
{
  double krl;
  
  krl = 1.0 - 2207*pow((1.0-sw),1.0121);
  
  return(krl);
}



/**
   function computes water relative permeability
   @param pw - water pressure
   @param pg - air pressure
   
   @retval krg - gas relative permeability

   12/9/2008, TKr
*/
double con_awf2mat::get_krg(double pw, double pg)
{
  double krg,sw,se;
  
  sw = get_sw(pw,pg);
  se = get_se(sw);
  krg = pow((1.0-se),2.0)*(1.0 - pow(se,(5.0/3.0)));
  
  return(krg);
}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kuw - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_kuw(double pw, double pg)
{
  double kuw;
  
  kuw = -alpha;

  return(kuw);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kwu - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_kwu(double pw, double pg)
{
  double kwu;

  kwu = 0.0;

  return(kwu);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kww - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_kww(double pw, double pg)
{
  double krl,sw,kww;
  //double krw,muw,kww;
  
  //krw = get_krw(pw);
  //kintr = get_kintr();
  //muw = get_muw();
  //kww = krw*kintr/muw;

  sw = get_sw(pw,pg);
  krl = get_krl(sw);
  kww = krl*kintr/muw;
  return(kww);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kwg - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_kwg(double pw, double pg)
{
  double kwg;
  
  kwg = 0.0;

  return(kwg);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kgw - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_kgw(double pw, double pg)
{
  double kgw;
  
  kgw = 0.0;

  return(kgw);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval kgg - conductivity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_kgg(double pw, double pg)
{
  double krg,kgg;
  //double krg,mug,kgg;
  
  //krg = get_krg(pw);
  //kintr = get_kintr();
  //mug = get_mug();
  //kgg = krg*kintr/mug;
  
  krg = get_krg(pw,pg);
  kgg = krg*kintr/mug;

  return(kgg);
}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capuw - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_capuw(double pw, double pg)
{
  double capuw;
  
  capuw = 0.0;

  return(capuw);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capwu - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_capwu(double pw, double pg)
{
  double capwu;
  
  capwu = alpha;

  return(capwu);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capww - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_capww(double pw, double pg)
{
  double sw,cs,n,capww;  
  
  sw = get_sw(pw,pg);
  cs = get_cs(pw,pg);
  n = phi0;
  
  capww = (alpha-n)/ks*sw*(sw - pw*cs/n + pg*cs/n) + n*sw/kw - cs;

  return(capww);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capwg - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_capwg(double pw, double pg)
{
  double sw,sg,cs,n,capwg;  
  
  sw = get_sw(pw,pg);
  sg = 1.0 - sw;
  cs = get_cs(pw,pg);
  n = phi0;
  
  capwg = (alpha-n)/ks*sw*(sg + pw*cs/n - pg*cs/n) + cs;

  return(capwg);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capgw - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_capgw(double pw, double pg)
{
  double sw,sg,cs,n,capgw;  
  
  sw = get_sw(pw,pg);
  sg = 1.0 - sw;
  cs = get_cs(pw,pg);
  n = phi0;
  
  capgw = (alpha-n)/ks*sg*(sw + pg*cs/n - pw*cs/n) + cs;

  return(capgw);
}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure
   @param pg - air pressure

   @retval capgg - capacity coefficient

   29/10/2009, TKr
*/
double con_awf2mat::get_capgg(double pw, double pg)
{
  double sw,sg,cs,n,capgg;  
  
  sw = get_sw(pw,pg);
  sg = 1.0 - sw;
  cs = get_cs(pw,pg);
  n = phi0;

  capgg = (alpha-n)/ks*sg*(sw - pg*cs/n + pw*cs/n) + n*sg/pg - cs;

  return(capgg);

}


/**
   function returns ... 
   @param pw - water pressure
   @param pg - air pressure

   @retval fw - first part for right-hand side for continutiy equation

   29/10/2009, TKr
*/
double con_awf2mat::get_fw1(double pw, double pg)
{
  double fw1,sw,krl;

  sw = get_sw(pw,pg);
  krl = get_krl(sw);
  
  fw1 = krl*kintr/muw*rhow;

  return(fw1);

}


/**
   function returns ... 
   @param pw - water pressure
   @param pg - air pressure

   @retval fg - first part for right-hand side for continutiy equation

   29/10/2009, TKr
*/
double con_awf2mat::get_fg1(double pw, double pg)
{
  double fg1,krg,rhog;

  krg = get_krg(pw,pg);
  rhog = 1.25;//docasne??!!

  fg1 = krg*kintr/mug*rhog;

  return(fg1);

}



/**
   function returns ... 
   @param pw - water pressure
   @param pg - air pressure

   @retval fu1 - first part for right-hand side for balance equation

   29/10/2009, TKr
*/
double con_awf2mat::get_fu1(double pw, double pg)
{
  double fu1;  
  
  fu1 = 0.0;

  return(fu1);

}


/**
     function prints names of all variables in nodes
     @param out - output rhle
     @param compother - number of other components
*/
void con_awf2mat::print_othervalue_name(FILE *out,long compother)
{
  switch (compother){
  case 0:{//capillary pressure
    fprintf (out,"Capillary pressure (Pa)");
    break;
  }
  default:{
   // fprintf (stderr,"\n\n unknown type of component is required in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}
