#include "dnnpsolvert.h"
#include "nnpsolvert.h"
#include "globalt.h"
#include "globmatt.h"
#include "transprint.h"
#include "saddpoint.h"
#include <string.h>

void solve_discont_nonlin_nonstationary_problem ()
{
  nonlin_nonstat_dform ();
}

/**
   function solves nonstationary transport problem with discontinuity
   the d-form version of the generalized trapezoidal method
   see T.J.R. Hughes: The Finite Element Method; Linear Static
   and Dynamic Finite Element Analysis

   JK, 19.8.2008
*/
void nonlin_nonstat_dform ()
{
  long i,j,k,n,nbdof,nsad,lcid,ani,ini,stop,nsts;
  long *dmcn;
  double *err,dt,dtmin,dtmax,end_time,alpha,zero,*thresh;
  double *f,*d,*p,*v,*z,*lhs,*lhsi,*tdlhs,*rhs,*lhsb,*tdlhsb;
  double *condmat,*condvect,*splhs,*sprhs;
  densemat dm;
  
  //  load case id must be equal to zero in this type of problem
  lcid=0;
  
  
  //  the number of primal unknowns / primal degrees of freedom
  //  it does not contain the number of multipliers
  n=Ndoft;
  //  the number of primal boundary/interface unknowns
  nbdof=Gtt->nbdof;
  //  the number of unknowns in reduced problem
  //  the sum of the number of primal boundary/interface unknowns and the number of Lagrange multipliers
  nsad=Gtt->nsad;
  
  //  computer zero
  zero=Tp->zero;
  //  threshold for the size of the right hand side
  thresh=Tp->threshrhs;
  
  //  nodal values
  lhs = Lsrst->give_lhs (0);
  //  time derivatives of nodal values
  tdlhs = Lsrst->give_tdlhs (0);
  //  initial nodal values
  lhsi = Lsrst->give_lhsi(0);
  //  prescribed fluxes / right hand side
  rhs = Lsrst->give_rhs (0);
  
  //  vector of prescribed fluxes (right hand side)
  f = new double [n];
  nullv (f,n);
  //  predictor
  d = new double [n];
  nullv (d,n);
  //  auxiliary vector
  p = new double [n];
  nullv (p,n);
  //  auxiliary vector
  v = new double [n];
  nullv (v,n);
  //  auxiliary vector
  z = new double [n];
  nullv (z,n);
  //  backup of nodal values
  lhsb = new double [n];
  nullv (lhsb,n);
  //  backup of time derivatives of nodal values
  tdlhsb = new double [n];
  nullv (tdlhs,n);


  
  //  array containing condensed matrix
  condmat = new double [nbdof*nbdof];
  nullv (condmat,nbdof*nbdof);
  //  array containing condensed vector
  condvect = new double [nbdof];
  nullv (condvect,nbdof);
  
  //  dense matrix for resulting matrix of the system
  dm.alloc (nsad);
  
  //  array containing solution of the saddle point problem
  splhs = new double [nsad];
  nullv (splhs,nsad);
  //  array containing the right hand side of the saddle point problem
  sprhs = new double [nsad];
  nullv (sprhs,nsad);

  //  code numbers of primal variables in the resulting system of equations
  dmcn = new long [nbdof];
  for (i=0;i<nbdof;i++){
    dmcn[i]=i+1;
  }

  
  
  //  nodes - integration points interpolation
  approximation ();

  // vlozil JM 15.1.2008
  // nulleqother ();
  if (Tp->nvs==1 && Tp->pnvs==1)
    actual_previous_nodval ();
  
  
  //  parameter of the trapezoidal method
  alpha=Tp->alpha;
  //  prescribed norm of residual
  err=Tp->errarr;
  //  maximum number of iterations in one time step
  ini=Tp->nii;
  //  minimum time increment
  dtmin=Tp->timecont.dtmin;
  
  //  maximum time increment
  dtmax=Tp->timecont.dtmax;

  //  initial time
  Tp->time=Tp->timecont.starttime ();
  //  end time
  end_time = Tp->timecont.endtime ();
  //  initial time increment
  dt = Tp->timecont.initialtimeincr ();
  
  // vlozil JM 25.4.2008----*****
  Tm->initmaterialmodels();
  //  nodes - integration points interpolation
  approximation (); 
  
  
  // nulleqother ();
  if (Tp->nvs==1 && Tp->pnvs==1)
    actual_previous_nodval ();
  // -----*******
 

  // ***************************
  //  main iteration loop  ****
  // ***************************
  //  number of time step
  i=0;
  //  number of successful time steps (number of steps without inner iterations)
  nsts=0;

  stop=0;

  Tm->initmaterialmodels();

  print_initt(-1, "wt");
  print_stept(0,i,Tp->time,NULL);

  do{
    
    //  time update
    Tp->time=Tp->timecont.newtime (dt);
    
    if (Mesprt != 0)  fprintf (stdout,"\n\n iteration number %ld, time %f, time step = %f\n",i,Tp->time,dt);
    
    Tm->updateipval ();
    i++;
    
    //  backup of attained nodal values and their time derivatives
    for (j=0;j<n;j++){
      lhsb[j]=lhs[j];
      tdlhsb[j]=tdlhs[j];
    }

    //  capacity matrix
    capacity_matrix (0);
    
    //  conductivity matrix
    conductivity_matrix (0);
    
    //  predictor d
    for (j=0;j<n;j++){
      d[j] = lhs[j]+dt*(1.0-alpha)*tdlhs[j];
    }

    //  C.d
    Cmat->gmxv (d,p);
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    
    //  prescribed fluxes f_{n+1}
    trfel_right_hand_side (lcid,f,n);
    
    //  C.d + alpha dt f_{n+1}
    for (j=0;j<n;j++){
      rhs[j] = f[j]*alpha*dt + p[j];
    }

    Tt->compute_jumps (rhs);
    nullv (z,n);
    Tt->compute_jumps (z);

    Gtt->mult_localization (Kmat);
    
    
    //Kmat->printmat (Outt);

    Tp->ssle->solve_system (Gtt,Kmat,lhs,rhs,Outt);
    
   /* fprintf (Outt,"\n\n kontrola reseni");
    for (j=0;j<n;j++){
      fprintf (Outt,"\n rhs %6ld   %20.15lf      lhs  %20.15lf",j,rhs[j],lhs[j]);
    }
    fprintf (Outt,"\n\n");
    */
    //Kmat->printmat (Outt);
    

    
    /*
    //  condensation of internal unknowns
    //  assembling of condensed matrix and vector
    Kmat->condense (Gtt,condmat,condvect,lhs,rhs,nbdof,1,Outt);
    
    
    //  localization of the condensed matrix to the resulting system of equations
    dm.localized (condmat,dmcn,nbdof);
    
    //  localization of the condensed vector to the resulting system of equations
    locglob (sprhs,condvect,dmcn,nbdof);
    
    //  localization of coupling blocks to the resulting system of equations
    //  nutno doplnit
    
    
    //  solution of the saddle point problem
    dm.gemp (splhs,sprhs,1,zero,1);
    
    
    //  localization of solution of the saddle point problem to the condensed vector
    globloc (splhs,condvect,dmcn,nbdof);
    
    //  back substitution
    Kmat->condense (Gtt,condmat,condvect,lhs,rhs,nbdof,2,Outt);
    */

    //  computation of time derivatives of the nodal values
    for (j=0;j<n;j++){
      tdlhs[j]=(lhs[j]-d[j])/dt/alpha;
    }
    
    //  computation of values at integration points from nodal values
    // approximation ();
    // *************************
    // vlozil JM 25.4.2008
    
    //  physically corrected solution
    solution_correction ();    
    //  approximation of nodal values into ontegration points
    approximation ();
    
    if (Tp->nvs==1 && Tp->pnvs==1)
      actual_previous_nodval ();
    // ***********************************
    




    //  iteration for equilibrium
    for (j=0;j<ini;j++){
      
      //  capacity matrix
      capacity_matrix (lcid);
      
      //  conductivity matrix
      conductivity_matrix (lcid);
      
      
      //  matrix of the system of equations
      //  C + alpha.dt.K
      Kmat->scalgm (dt*alpha);
      Kmat->addgm (1.0,*Cmat);
      
      //  (C + alpha.dt.K).d
      Gtt->mult_localization (Kmat);
      nullv (v,n);
      Kmat->gmxv (lhs,v);
      
      //  C.d
      nullv (p,n);
      Cmat->gmxv (d,p);
      
      //  computation of residuals
      //norrhs=0.0;
      //norres=0.0;
      
      for (k=0;k<n;k++){
	rhs[k]=f[k]*alpha*dt+p[k]-v[k];
	//	fprintf (stdout,"\n rhs   %15.10le   k je %d",rhs[k],k);
	//norres+=rhs[k]*rhs[k];
	z[k]+=f[k]*alpha*dt+p[k];
	//norrhs+=z[k]*z[k];
	//fprintf (stdout,"\n rhs   %15.10le   z  %15.10le  v %15.10le",rhs[k],z[k],v[k]);
      }
      //norres=sqrt(norres);
      //norrhs=sqrt(norrhs);

      //fprintf (stdout,"\n KONTROLA norres   %15.10le",norres);

      Tt->compute_jumps (rhs);
      
      stop = norm_computation_vec (rhs,z,err,thresh,2,1);
      
      
      nullv (z,n);
      Tt->compute_jumps (z);
      //fprintf (stdout,"\n pauza \n");
      
      if (stop==1){
	break;
      }
      
      /*
      fprintf (stdout,"\n norres   %15.10le, nrhs   %15.10le,   norres/norrhs  %15.10le",norres,norrhs,norres/norrhs);
      
      if (norres/norrhs<err){
	break;
      }
      */

      /*
      if (Mesprt != 0)  
	fprintf (stdout,"\n nres   %15.10le, nrhs   %15.10le",norres,norrhs);
      
      if (norrhs<Tp->zero){
	if (norres<err){
	  break;
	}
      }
      else{
	if (Mesprt != 0)  
	  fprintf (stdout,"   nres/nrhs   %15.10le",norres/norrhs);
	if (norres/norrhs<err){
	  break;
	}
      }
      */
      

      if (Mesprt != 0)  fprintf (stdout,"\n iteration number %ld",j);
      
      /*
      nullv (rhs,n);
      for (k=0;k<n;k++){
	rhs[k]=f[k]*alpha*dt+p[k];
      }
      
      Tt->compute_jumps (rhs);
      */

      Tp->ssle->solve_system (Gtt,Kmat,z,rhs,Outt);
      
      for (k=0;k<n;k++){
	lhs[k]+=z[k];
      }
      
      nullv (z,n);
      Tt->compute_jumps (z);
      
      //  computation of time derivatives of the nodal values
      for (k=0;k<n;k++){
	tdlhs[k]=(lhs[k]-d[k])/dt/alpha;
      }
      
      //  computation of values at integration points from nodal values
      // approximation ();
      
      // *************************
      // vlozil JM 25.4.2008
      
      //  physically corrected solution
      solution_correction ();    
      //  approximation of nodal values into ontegration points
      approximation ();
      
      
      // nulleqother ();
      if (Tp->nvs==1 && Tp->pnvs==1)
	actual_previous_nodval ();
      // ***********************************
      
    }
    
    //  actual number of perfromed iterations
    ani=j;
    
    if (ani==0)
      nsts++;
    
    if (ani==ini){
      //  backup is used for actual values
      for (j=0;j<n;j++){
	lhs[j]=lhsb[j];
	tdlhs[j]=tdlhsb[j];
      }
      
      // *************************
      // vlozil JM 25.4.2008
      
      //  physically corrected solution
      solution_correction ();    
      //  approximation of nodal values into ontegration points
      approximation ();
      
      
      
      // nulleqother ();
      if (Tp->nvs==1 && Tp->pnvs==1)
	actual_previous_nodval ();
      // ***********************************
      
      
      //  reduction of the time increment because
      //  inner loop was not able to enforce equilibrium
      dt/=2.0;
      i--;
      Tp->timecont.oldtime ();

      //fprintf (stdout,"\n time increment is reduced because the inner loop was not able to enforce equilibrium");
      fprintf (stdout,"\n time increment is reduced because the inner loop was not able to enforce equilibrium dt = %le",dt);
      //  fprintf (Outt2,"\n time increment is reduced because the inner loop was not able to enforce equilibrium");
      if (dt<dtmin){
	fprintf (stderr,"\n\n time increment is less than minimum time increment");
	fprintf (stderr,"\n computation fails (file %s, line %d)\n",__FILE__,__LINE__);
	abort ();
      }
    }else{
      //  time increment
      //Tp->time = Tp->timecont.newtime ();
      //dtdef = Tp->timecont.actualforwtimeincr ();
      
      if (nsts==2){
		  dt*=1.2;
		  if (dt>dtmax) {
		  dt = dtmax;

		  }
	//dt*=1.2;
	nsts=0;
	//	fprintf (stdout,"\n time increment is enlarged because no inner loop was neccessary in previous 3 steps");
	//fprintf (stdout,"\n time increment is enlarged because no inner loop was neccessary in previous 3 steps dt = %le, dtdef = %le",dt,dtdef);
	fprintf (stdout,"\n time increment is enlarged because no inner loop was neccessary in previous 3 steps dt = %le",dt);
	//fprintf (Outt2,"\n time increment is enlarged because no inner loop was neccessary in previous 3 steps dt= %lf",dt);
      }
      
      //if (dt>dtdef)
      //dt=dtdef;
      
      //Tp->time = Tp->timecont.newtime (dt);
      
      
      //time+=dt;
      //Tp->time=time;
      //Tp->timecont.time=time;
      //Tp->timecont.fdt=dt;
      
      print_stept(0,i,Tp->time,NULL);
      print_flusht();
    }

  }while(Tp->time<end_time);
  
  delete [] dmcn;
  delete [] sprhs;
  delete [] splhs;
  delete [] condvect;
  delete [] condmat;
  delete [] v;
  delete [] p;
  delete [] d;
  delete [] f;
  delete [] lhsb;
  delete [] tdlhsb;
  delete [] z;
  delete [] err;
  delete [] thresh;
  delete [] lhs;
  delete [] lhsi;
  delete [] tdlhs;
  delete [] rhs;



  print_closet();
}
