#include "globalt.h"
#include "saltmat4.h"
#include "kunzel.h"

kunmat *kun2; 

/**********************************************
*                                             *
* General material model for 3 media transfer *
*      9 independent parameters               *
***********************************************/
/* x1 (w) is the volumetric water content [m3/m3]
   x2  (Cf) is the concentration of free ions in water [kg/m3]
   x3  (Cfyz)
   x4  (T) is the temperature


   D the ion diffusivity [m2/s]
   Kapa = the moisture diffusivity [m2/s]

 position CORD:     2 - density
                    3 - porosity
                    4 - faktor difusniho odporu
                    5 - kapa
                    6 - sorption izoterm
                    7 - saturated moisture
                    8 - none
                    9 - Cecko
                    10 - Lambda
                    11 - 13 - none
                    14 - Dcoef
                    15 - binding isotherm
                    16 - cfmax
                    17 - ws
                    18 - 19 - none

*/

FILE *in1a,*in2a,*in3a,*in4a;

saltmat4::saltmat4 ()
{
  mw = 0.01801528; //molar mass of water kg.mol-1
  ma = 28.9645;   //molar mass of dry air kg.mol-1
  gasr = 8.31441; //universal gas constant J.mol-1.K-1

}

saltmat4::~saltmat4 ()
{}


/**
   function computes conductivity matrix of the material
   in the required integration point
   
   @param d - conductivity %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

*/
void saltmat4::matcond (matrix &d,long ri,long ci,long ipp)
{
  long n;
  n = d.n;
  
  switch (n){
  case 1:{
    matcond1d (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    matcond3d (d,ri,ci,ipp);//3D
    break;
  }
  case 4:{
    matcond4d (d,ri,ci,ipp);//4D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}

/**
   function computes conductivity matrix of the material
   in the required integration point
   
   @param d - conductivity %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point

*/
void saltmat4::matcond2 (matrix &d,long ri,long ci,long ipp)
{
  long n;
  n = d.n;

  switch (n){
  case 1:{
    //matcond1d (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d2 (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    //matcond3d (d,ri,ci,ipp);//3D
    break;
  }
  case 4:{
    //matcond4d (d,ri,ci,ipp);//4D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}

/**
   function creates conductivity matrix of the material for 1D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void saltmat4::matcond1d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double x1,x2,x3, x4,cfmax;
  k = 0.0;
  
  x1 = Tm->ip[ipp].av[0];
  x2 = Tm->ip[ipp].av[1];
  x3 = Tm->ip[ipp].av[2];
  x4 = Tm->ip[ipp].av[3];
  
  cfmax=Tm->ip[ipp].eqother[6];

  if (x2 >= cfmax){
       x2 = cfmax;	
  }
  
    
  if((ri == 0) && (ci == 0))
    k = k11 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 1))
    k = k12 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 2))
    k = k13 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 3))
    k = k14 (x1,x2,x3,x4,ipp);

  if((ri == 1) && (ci == 0))
    k = k21 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 1))
    k = k22 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 2))
    k = k23 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 3))
    k = k24 (x1,x2,x3,x4,ipp);

  if((ri == 2) && (ci == 0))
    k = k31 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 1))
    k = k32 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 2))
    k = k33 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 3))
    k = k34 (x1,x2,x3,x4,ipp);

  if((ri == 3) && (ci == 0))
    k = k41 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 1))
    k = k42 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 2))
    k = k43 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 3))
    k = k44 (x1,x2,x3,x4,ipp);
  
  d[0][0] = k;
}

void saltmat4::matcond2d2 (matrix &d,long ri,long ci,long ipp)
{
  d[0][0] = 0.0;  d[0][1] = 0.0;
}

/**
   function creates conductivity matrix of the material for 2D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void saltmat4::matcond2d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double x1,x2,x3, x4,cfmax;
  k = 0.0;
  
  x1 = Tm->ip[ipp].av[0];
  x2 = Tm->ip[ipp].av[1];
  x3 = Tm->ip[ipp].av[2];
  x4 = Tm->ip[ipp].av[3];
  
  cfmax=Tm->ip[ipp].eqother[6];

  if (x2 >= cfmax){
    x2 = cfmax;
  }

  if((ri == 0) && (ci == 0))
    k = k11 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 1))
    k = k12 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 2))
    k = k13 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 3))
    k = k14 (x1,x2,x3,x4,ipp);

  if((ri == 1) && (ci == 0))
    k = k21 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 1))
    k = k22 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 2))
    k = k23 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 3))
    k = k24 (x1,x2,x3,x4,ipp);

  if((ri == 2) && (ci == 0))
    k = k31 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 1))
    k = k32 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 2))
    k = k33 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 3))
    k = k34 (x1,x2,x3,x4,ipp);

  if((ri == 3) && (ci == 0))
    k = k41 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 1))
    k = k42 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 2))
    k = k43 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 3))
    k = k44 (x1,x2,x3,x4,ipp);

  fillm(0.0,d);
  
  d[0][0] = k;   d[0][1] = 0.0;
  d[1][0] = 0.0; d[1][1] = k;

}

/**
   function creates conductivity matrix of the material for 3D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/

void saltmat4::matcond3d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double x1,x2,x3, x4,cfmax;
  k = 0.0;
  
  x1 = Tm->ip[ipp].av[0];
  x2 = Tm->ip[ipp].av[1];
  x3 = Tm->ip[ipp].av[2];
  x4 = Tm->ip[ipp].av[3];
    
  cfmax=Tm->ip[ipp].eqother[6];

  if (x2 >= cfmax){
  //  x2 = cfmax;
  }

  if((ri == 0) && (ci == 0))
    k = k11 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 1))
    k = k12 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 2))
    k = k13 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 3))
    k = k14 (x1,x2,x3,x4,ipp);

  if((ri == 1) && (ci == 0))
    k = k21 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 1))
    k = k22 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 2))
    k = k23 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 3))
    k = k24 (x1,x2,x3,x4,ipp);

  if((ri == 2) && (ci == 0))
    k = k31 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 1))
    k = k32 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 2))
    k = k33 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 3))
    k = k34 (x1,x2,x3,x4,ipp);

  if((ri == 3) && (ci == 0))
    k = k41 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 1))
    k = k42 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 2))
    k = k43 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 3))
    k = k44 (x1,x2,x3,x4,ipp);
  
  fillm(0.0,d);
  
  d[0][0]=k;   d[0][1]=0.0; d[0][2]=0.0;
  d[1][0]=0.0; d[1][1]=k;   d[1][2]=0.0;
  d[2][0]=0.0; d[2][1]=0.0; d[2][2]=k;
}


void saltmat4::matcond4d (matrix &d,long ri,long ci,long ipp)
{
  double k;
  double x1,x2,x3, x4,cfmax;
  k = 0.0;
  
  x1 = Tm->ip[ipp].av[0];
  x2 = Tm->ip[ipp].av[1];
  x3 = Tm->ip[ipp].av[2];
  x4 = Tm->ip[ipp].av[3];
    
 
  cfmax=Tm->ip[ipp].eqother[6];

  if (x2 >= cfmax){
    //x2 = cfmax;
  }

  if((ri == 0) && (ci == 0))
    k = k11 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 1))
    k = k12 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 2))
    k = k13 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 3))
    k = k14 (x1,x2,x3,x4,ipp);

  if((ri == 1) && (ci == 0))
    k = k21 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 1))
    k = k22 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 2))
    k = k23 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 3))
    k = k24 (x1,x2,x3,x4,ipp);

  if((ri == 2) && (ci == 0))
    k = k31 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 1))
    k = k32 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 2))
    k = k33 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 3))
    k = k34 (x1,x2,x3,x4,ipp);

  if((ri == 3) && (ci == 0))
    k = k41 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 1))
    k = k42 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 2))
    k = k43 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 3))
    k = k44 (x1,x2,x3,x4,ipp);
  
  fillm(0.0,d);
  
  d[0][0]=k;   d[0][1]=0.0; d[0][2]=0.0; d[0][3]=0.0;
  d[1][0]=0.0; d[1][1]=k;   d[1][2]=0.0; d[1][3]=0.0;
  d[2][0]=0.0; d[2][1]=0.0; d[2][2]=k;   d[2][3]=0.0;
  d[3][0]=0.0; d[3][1]=0.0; d[3][2]=0.0; d[3][3]=k;
}

/**
   function creates capacity matrix of the material

   @param c   - capacity matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void saltmat4::matcap (double &c,long ri,long ci,long ipp)
{
  double x1,x2,x3,x4, cfmax;
  c = 0.0;
  
  x1 = Tm->ip[ipp].av[0];
  x2 = Tm->ip[ipp].av[1];
  x3 = Tm->ip[ipp].av[2];
  x4 = Tm->ip[ipp].av[3];

  cfmax=Tm->ip[ipp].eqother[6];

  if (x2 >= cfmax){
    //x2 = cfmax;
  }
    
  if((ri == 0) && (ci == 0))
    c = c11 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 1))
    c = c12 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 2))
    c = c13 (x1,x2,x3,x4,ipp);
  if((ri == 0) && (ci == 3))
    c = c14 (x1,x2,x3,x4,ipp);

  if((ri == 1) && (ci == 0))
    c = c21 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 1))
    c = c22 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 2))
    c = c23 (x1,x2,x3,x4,ipp);
  if((ri == 1) && (ci == 3))
    c = c24 (x1,x2,x3,x4,ipp);

  if((ri == 2) && (ci == 0))
    c = c31 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 1))
    c = c32 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 2))
    c = c33 (x1,x2,x3,x4,ipp);
  if((ri == 2) && (ci == 3))
    c = c34 (x1,x2,x3,x4,ipp);

  if((ri == 3) && (ci == 0))
    c = c41 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 1))
    c = c42 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 2))
    c = c43 (x1,x2,x3,x4,ipp);
  if((ri == 3) && (ci == 3))
    c = c44 (x1,x2,x3,x4,ipp);
}


void saltmat4::values_correction (vector &nv,long ipp)
{
  double cfmax;
  double a,n;
  int kod;
  double smc,x1;

  x1 = nv[0];
  CorD(7,kod,1,0,x1,smc,a,n);
  
  cfmax=Tm->ip[ipp].eqother[6];
// je to jen pro urcity material... nutno opravit... 25.ledna 2008
  if (nv[1] >= cfmax){
//	fprintf(Outt, "%le  %le  %ld\n",  Tp->time, nv[1],ipp);// kapa a decko
    nv[1] = cfmax;
  }
 /* if(nv[0] < 0){
	  nv[0] = 1e-6;
  }
  if(nv[0] > smc){
	  nv[0] = smc;
  }
  if(nv[2] < 0){
	  fprintf(Outt, " Cfkryst %le  %le  %ld\n",  Tp->time, nv[2],ipp);// kapa a decko
	  nv[2] = 0.0;
  }*/
}



/**
   function reads data and material parameters

   @param in  - input file
*/
void saltmat4::read (XFILE *in)
{
  int i,j;
  for (i = 2; i<19; i++)
    {
      xfscanf(in, "%d",&MatChar[i]);
    }
  xfscanf(in, "%d",&MatChar[19]);
  for (i = 2;i <= 19; i++)
    {
      
      switch (MatChar[i])
	{
	case 0:
	  break;
	case 1:xfscanf(in, "%lf",&MatConst[i]);
	  //edit_chechbox(i,1);
	  break;
	case 2:   if( MatChar[i-1] ==2)
			  {
				  xfscanf(in, "%lf %lf",&MatData[i][0][0],&MatData[i][1][0]);
			  }
		else
		{
			xfscanf(in, "%lf %lf",&MatData[i][0][0],&MatData[i][1][0]);
		}
		switch (int(MatData[i][1][0])){
			case 2: for (j=1; j<=MatData[i][0][0];j++)
					{
						xfscanf(in, "%lf %lf",&MatData[i][0][j],&MatData[i][1][j] );
					}
				break;
			case 3: for (j=1; j<=MatData[i][0][0];j++)
					{
						xfscanf(in, "%lf %lf %lf",&MatData[i][0][j],&MatData[i][1][j],&MatData[i][2][j] );
					}
				break;
		}
		xfscanf(in, "%d",&MatData[i][0][0]);
		break;
	case 3://edit_chechbox(i,3);
	  break;
	case 30:   if( MatChar[i-1] ==1)
	    {
	      xfscanf(in, "%lf %lf",&MatFunce[i][0],&MatFunce[i][1]);
	    }
	  else
	    {
	      xfscanf(in, "%lf %lf",&MatFunce[i][0],&MatFunce[i][1]);
	    }
	  
	  break;
	case 31: if( MatChar[i-1] ==1)
	    {
	      if (i == 6)
		{
		  xfscanf(in, "%lf %lf %lf",&MatFunce[i][0],&MatFunce[i][1], &MatFunce[i][2]);
		}
	      else
		{
		  xfscanf(in, "%lf %lf",&MatFunce[i][0],&MatFunce[i][1]);
		}
	    }
	  else
	    {
	      if (i == 6)
		{
		  xfscanf(in, "%lf %lf %lf",&MatFunce[i][0],&MatFunce[i][1], &MatFunce[i][2]);
		}
	      else
		{
		  xfscanf(in, "%lf %lf",&MatFunce[i][0],&MatFunce[i][1]);
		}
	    }
	  
	  break;
	case 32:  if( MatChar[i-1] ==1){
		xfscanf(in, "%lf %lf",&MatFunce[i][0],&MatFunce[i][1]);
			  }
		else
		{
			xfscanf(in, "%lf %lf",&MatFunce[i][0],&MatFunce[i][1]);
		}
		break;
	case 40:{
		xfscanf(in, "%lf %lf",&MatData[i][0][0],&Init[i]);
		xfscanf(in, "%lf %lf %lf %lf %lf %lf",&k1[i], &k2[i], &k3[i],&k4[i], &k5[i], &k6[i]);
		for (j=1; j<=MatData[i][0][0];j++)
		{
			xfscanf(in, "%lf %lf",&MatData[i][0][j],&MatData[i][1][j]);
		}
		xfscanf(in, "%lf",&MatData[i][0][0]);
		xfscanf(in, "%lf",&MatData[i][2][0]);
		for (j=1; j<=MatData[i][2][0];j++)
		{
			xfscanf(in, "%lf %lf",&MatData[i][2][j] ,&MatData[i][3][j] );
		}
		xfscanf(in, "%lf",&MatData[i][2][0]);
		break;
			}
	  }
  }
}


/******************************************************
Conductivity and capacity terms for C and K matrices 
*******************************************************/

double saltmat4::k11(double x1,double x2,double x3,double x4,long ipp)
{
  double k11;
  double kapak, ps, delta, dfdw;
  double a,n;
  int kod;
  double smc;
  CorD(7,kod,1,0,x1,smc,a,n);

  if( x1 > smc) x1 = smc;

  kapak=Tm->ip[ipp].eqother[0];
 
  // ps = pgws(t);
  ps = pgws(x1,x2,x3,x4);
  //delta = permeabilitavodnipary(t,p);
  delta = permeabilitavodnipary(x1,x2,x3,x4);
  
  //sorption_izoterms_giva_data(1,x1,x2,x3,x4,a,dfdw,ipp); // jeste doladit 
  //fi = Tm->ip[ipp].eqother[5];
  dfdw = Tm->ip[ipp].eqother[2];



  //CorD(6,kod,1,1,x1,u,a,n);
/*
  switch (kod)
  {
  case 2:   //fi = u;
	  dfdw = a;
	  break;
  case 30:
	  break;
  case 31:  {
	  dfdw = (u/(a*x1*n))*pow((1-(log(x1))/a),(-1-(1/n)));
			}
	  break;
  }*/
  
  k11 = 1000*kapak + ps*delta*dfdw;
  return (k11);
}

double saltmat4::k12(double x1,double x2,double x3,double x4,long ipp)
{
  double k12;
   double dcoef;
   double smc,a,n;
   int kod;
   CorD(7,kod,1,0,x1,smc,a,n);

  if( x1 > smc) x1 = smc;

   dcoef=Tm->ip[ipp].eqother[3];
   k12 = -dcoef*x1;

  return (k12);
}

double saltmat4::k13(double x1,double x2,double x3,double x4,long ipp)
{
  double k13;

  k13 = 0.0;

  return (k13);
}

double saltmat4::k14(double x1,double x2,double x3,double x4,long ipp)
{
  double k14, delta,dpvsdt;
  double u;

  delta = permeabilitavodnipary(x1,x2,x3,x4);
  //sorption_izoterms_giva_data(0,x1,x2,x3,x4,u,a, ipp); // jeste doladit

   u = Tm->ip[ipp].eqother[1];

  //CorD(6,kod,1,1,x1,u,a,n);
  dpvsdt =  derivation_saturation_water_vapour_pressure_temperature(x1,x2,x3,x4);
  
  k14 = delta*u*dpvsdt;

  return (k14);
}

double saltmat4::k21(double x1,double x2,double x3,double x4,long ipp)
{
  double k21;
  double kapak;

  kapak=Tm->ip[ipp].eqother[0];
  k21 = kapak*x2*1;
  
  return (k21);
}

double saltmat4::k22(double x1,double x2,double x3,double x4,long ipp)
{
  double k22;
  double dcoef;

   double smc,a,n;
   int kod;
  CorD(7,kod,1,0,x1,smc,a,n);

  if( x1 > smc) x1 = smc;

  dcoef=Tm->ip[ipp].eqother[3];
  k22 = dcoef*x1;

  return (k22);
}

double saltmat4::k23(double x1,double x2,double x3,double x4,long ipp)
{
  double k23;

  k23 = 0.0;
  
  return (k23);
}

double saltmat4::k24(double x1,double x2,double x3,double x4,long ipp)
{
  double k24;

  k24 = 0.0;
  
  return (k24);
}

double saltmat4::k31(double x1,double x2,double x3,double x4,long ipp)
{
  double k31;

  k31 = 0.0;

  return (k31);
}

double saltmat4::k32(double x1,double x2,double x3,double x4,long ipp)
{
  double k32;

  k32= 0.0;

  return (k32);
}

double saltmat4::k33(double x1,double x2,double x3,double x4,long ipp)
{
  double k33;

  k33= 0.0;

  return (k33);
}

double saltmat4::k34(double x1,double x2,double x3,double x4,long ipp)
{
  double k34;

  k34= 0.0;

  return (k34);
}

double saltmat4::k41(double x1,double x2,double x3,double x4,long ipp)
{
  double k41;
  double smc, lv1, ps, dp,dfdw , a,n;
  int kod;
  
  CorD(7,kod,1,0,x1,smc,a,n);

  if( x1 > smc) x1 = smc;
  
  
  lv1 = latent_heat_of_evaporation_of_water (x1,x2,x3,x4);
  ps = pgws (x1,x2,x3,x4);
  dp = permeabilitavodnipary(x1,x2,x3,x4);
  
  /*CorD(6,kod,1,1,x1,u,a,n);

  switch (kod)
  {
  case 2:   //fi = u;
	  dfdw = a;
	  break;
  case 30:
	  break;
  case 31:  {
	  dfdw = (u/(a*x1*n))*pow((1-(log(x1))/a),(-1-(1/n)));
			}
	  break;
  }
  */
   //sorption_izoterms_giva_data(1,x1,x2,x3,x4,a,dfdw); // jeste doladit
   dfdw = Tm->ip[ipp].eqother[2];
     
  k41= lv1*ps*dp*dfdw;

  return (k41);
}

double saltmat4::k42(double x1,double x2,double x3,double x4,long ipp)
{
  double k42;

  k42= 0.0;

  return (k42);
}

double saltmat4::k43(double x1,double x2,double x3,double x4,long ipp)
{
  double k43;

  k43= 0.0;

  return (k43);
}

double saltmat4::k44(double x1,double x2,double x3,double x4,long ipp)
{
  double k44;


  double lv1, lambda, dpvsdt, dp, a, n;
  double smc;
  int kod;

  CorD(7,kod,1,0,x1,smc,a,n);

  if( x1 > smc) x1 = smc;


  lv1 = latent_heat_of_evaporation_of_water (x1,x2,x3,x4);
  dp = permeabilitavodnipary(x1,x2,x3,x4);
  CorD(10,kd,1,0,x1,lambda,a2,a3);//lambda
  dpvsdt =  derivation_saturation_water_vapour_pressure_temperature(x1,x2,x3,x4);

  k44= lambda + lv1 * dp * dpvsdt;

  return (k44);
}


double saltmat4::c11(double x1,double x2,double x3,double x4,long ipp)
{
  double c11;
  double ps;
  double fi,dfdw, wsoli, por;
  double a, n;
  int kod;
  double smc;
  CorD(7,kod,1,0,x1,smc,a,n);

  if( x1 > smc) x1 = smc;


  ps = pgws(x1,x2,x3,x4);
  CorD(3,kd,1,0,x1,por,a2,a3);
  
   //sorption_izoterms_giva_data(0,x1,x2,x3,x4,fi,dfdw); // jeste doladit
   //sorption_izoterms_giva_data(1,x1,x2,x3,x4,a,dfdw); // jeste doladit
   
  fi = Tm->ip[ipp].eqother[1];
  
  dfdw = Tm->ip[ipp].eqother[2];
  
   /*CorD(6,kod,1,1,x1,u,a,n);

  switch (kod)
 {
  case 2:
	  fi = u;
	  dfdw = a;
	  break;
  case 30:
	  break;
  case 31:  {
	  dfdw = (u/(a*x1*n))*pow((1-(log(x1))/a),(-1-(1/n)));
	  fi = u*pow((1-(log(x1))/a),(-1/n));
	  //   printf ("\n w je %lf  fi je %lf a derivace je %lf",x1,fi, dfdw);
			}
	  break;
  }*/

 // if (ipp == 2) fprintf (Outt,"\n w je %lf  fi je %lf a derivace je %lf",x1,fi, dfdw);

  CorD(17,kd,0,0,x1,wsoli,a2,a3);

  c11 = 1000 + mw/(gasr*294.15)*ps*(por-x1-wsoli)*dfdw-mw*ps*fi/(gasr*294.15);

  return(c11);
}

double saltmat4::c12(double x1,double x2,double x3,double x4,long ipp)
{
  double c12;

  c12 = 0.0;

  return(c12);
}

double saltmat4::c13(double x1,double x2,double x3,double x4,long ipp)
{
  double c13;

  c13 = 0.0;

  return(c13);
}

double saltmat4::c14(double x1,double x2,double x3,double x4,long ipp)
{
  double c14;

  c14 = 0.0;

  return(c14);
}

double saltmat4::c21(double x1,double x2,double x3,double x4,long ipp)
{
  double c21,cfmax;

  cfmax=Tm->ip[ipp].eqother[6];

  if (x2 >= cfmax)
  {
	  c21 = cfmax;
	//  c21 = x2;
  }
  else
  {
    c21 = x2;
  }
  
  return(c21);
}

double saltmat4::c22(double x1,double x2,double x3,double x4,long ipp)
{
  double c22;
  double dcbdcf;
   double smc,a,n;
   int kod;
  CorD(7,kod,1,0,x1,smc,a,n);

  if( x1 > smc) x1 = smc;

 
  dcbdcf=Tm->ip[ipp].eqother[5];
  c22 = x1 + dcbdcf;
 
  return(c22);
}

double saltmat4::c23(double x1,double x2,double x3,double x4,long ipp)
{
  double c23;

  c23 = 1.0;

  return(c23);
}

double saltmat4::c24(double x1,double x2,double x3,double x4,long ipp)
{
  double c24;

  c24 = 0.0;

  return(c24);
}

double saltmat4::c31(double x1,double x2,double x3,double x4,long ipp)
{
  double c31;
  double cfmax;

  cfmax=Tm->ip[ipp].eqother[6];
  
  if (x2 < cfmax){
    c31 = 0.0;
  }
  else{
    c31 = cfmax-x2;
  }
  
  return(c31);
}

double saltmat4::c32(double x1,double x2,double x3,double x4,long ipp)
{
  double c32;
  double cfmax;
   double smc,a,n;
   int kod;
  CorD(7,kod,1,0,x1,smc,a,n);

  if( x1 > smc) x1 = smc;


  cfmax=Tm->ip[ipp].eqother[6];
  
  if (x2 < cfmax){
    c32 = 0.0;
  }
  else{
    c32 = -x1;
  }
  
  return(c32);
}

double saltmat4::c33(double x1,double x2,double x3,double x4,long ipp)
{
  double c33;

  c33 = 1.0;

  return(c33);
}

double saltmat4::c34(double x1,double x2,double x3,double x4,long ipp)
{
  double c34;

  c34 = 0.0;

  return(c34);
}

double saltmat4::c41(double x1,double x2,double x3,double x4,long ipp)
{
  double c41;

  c41 = 0.0;

  return(c41);
}

double saltmat4::c42(double x1,double x2,double x3,double x4,long ipp)
{
  double c42;

  c42 = 0.0;

  return(c42);
}
double saltmat4::c43(double x1,double x2,double x3,double x4,long ipp)
{
  double c43;

  c43 = 0.0;

  return(c43);
}
double saltmat4::c44(double x1,double x2,double x3,double x4,long ipp)
{
  double c44;
  double rom, c,a2,a3, smc;
  int kod;
  
  CorD(7,kod,1,0,x1,smc,a2,a3);

  if( x1 > smc) x1 = smc;

  if (x1 < 0) x1 = 0;

  CorD(2,kd,0,0,0,rom,a2,a3);
  CorD(9,kd,smc,0,x1,c,a2,a3);
  
  c44 = rom * c;

  return(c44);
}

/* Function calculates all auxiliary data - optional */
void saltmat4::auxiliarydata (double x1,double x2,double x3,double x4)
{}


/******************
Boundary conditions
*******************/
	
/**
   function computes new transmission coefficient (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param trc     - prescribed transmission coefficient on the boundary
   @param ri      - row index
   @param ci      - column index
   @param nn      - number of node
   @param bc      - type of boundary condition
   @param ipp     - number of first integration point on element
*/
double saltmat4::transmission_transcoeff(double trc,long ri,long ci,long nn,long bc,long ipp)
{
  long k;
  double new_trc,x1,x2,x3,x4;
  new_trc = 0.0;
  
  k=Gtt->give_dof(nn,0);
  if (k>0)   {x1 = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {x1 = 0.0;}
  if (k<0)   {x1 = Tb->lc[0].pv[0-k-1].getval();}
  k=Gtt->give_dof(nn,1);
  if (k>0)   {x2 = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {x2 = 0.0;}
  if (k<0)   {x2 = Tb->lc[0].pv[0-k-1].getval();}
  k=Gtt->give_dof(nn,2);
  if (k>0)   {x3 = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {x3 = 0.0;}
  if (k<0)   {x3 = Tb->lc[0].pv[0-k-1].getval();}
   k=Gtt->give_dof(nn,3);
  if (k>0)   {x4 = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {x4 = 0.0;}
  if (k<0)   {x4 = Tb->lc[0].pv[0-k-1].getval();}
  
  if((ri == 0) && (ci == 0))
    new_trc = get_transmission_transcoeff_11(x1,x2,x3,x4,bc,ipp);
  if((ri == 0) && (ci == 1))
    new_trc = 0.0;
  if((ri == 0) && (ci == 2))
    new_trc = 0.0;
  if((ri == 0) && (ci == 3))
    new_trc = 0.0;
  

  if((ri == 1) && (ci == 0))
    new_trc = 0.0;
  if((ri == 1) && (ci == 1))
    new_trc = 0.0;
  if((ri == 1) && (ci == 2))
    new_trc = 0.0;
  if((ri == 1) && (ci == 2))
    new_trc = 0.0;
  if((ri == 1) && (ci == 3))
    new_trc = 0.0;
  

  if((ri == 2) && (ci == 0))
    new_trc = 0.0;
  if((ri == 2) && (ci == 1))
    new_trc = 0.0;
  if((ri == 2) && (ci == 2))
    new_trc = 0.0;
  if((ri == 2) && (ci == 3))
    new_trc = 0.0;

  if((ri == 3) && (ci == 0))
    new_trc = 0.0;
  if((ri == 3) && (ci == 1))
    new_trc = 0.0;
  if((ri == 3) && (ci == 2))
    new_trc = 0.0;
  if((ri == 3) && (ci == 3))
    new_trc = 0.0;

  new_trc = new_trc*trc;

  return (new_trc);
}

/**
   function computes new nodal value (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param nodval     - prescribed nodal value
   @param ri         - row index
   @param ci         - column index
   @param nn         - number of node
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
double saltmat4::transmission_nodval(double nodval,long ri,long ci,long nn,long bc,long ipp)
{
  long k;
  double new_nodval,x1,x2,x3,x4;
  new_nodval = 0.0;
  
  k=Gtt->give_dof(nn,0);
  if (k>0)   {x1 = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {x1 = 0.0;}
  if (k<0)   {x1 = Tb->lc[0].pv[0-k-1].getval();}
  k=Gtt->give_dof(nn,1);
  if (k>0)   {x2 = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {x2 = 0.0;}
  if (k<0)   {x2 = Tb->lc[0].pv[0-k-1].getval();}
  k=Gtt->give_dof(nn,2);
  if (k>0)   {x3 = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {x3 = 0.0;}
  if (k<0)   {x3 = Tb->lc[0].pv[0-k-1].getval();}
  k=Gtt->give_dof(nn,3);
  if (k>0)   {x4 = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {x4 = 0.0;}
  if (k<0)   {x4 = Tb->lc[0].pv[0-k-1].getval();}
 
  if((ri == 0) && (ci == 0))
    new_nodval = get_transmission_nodval_11(nodval,x1,x2,x3,x4,bc,ipp);
  if((ri == 0) && (ci == 1))
    new_nodval = 0.0;
  if((ri == 0) && (ci == 2))
    new_nodval = 0.0;
if((ri == 0) && (ci == 3))
    new_nodval = 0.0;
  
  if((ri == 1) && (ci == 0))
    new_nodval = 0.0;
  if((ri == 1) && (ci == 1))
    new_nodval = 0.0;
  if((ri == 1) && (ci == 2))
    new_nodval = 0.0;
  if((ri == 1) && (ci == 3))
    new_nodval = 0.0;
 
  if((ri == 2) && (ci == 0))
    new_nodval = 0.0;
  if((ri == 2) && (ci == 1))
    new_nodval = 0.0;
  if((ri == 2) && (ci == 2))
    new_nodval = 0.0;
  if((ri == 2) && (ci == 3))
    new_nodval = 0.0;

  if((ri == 3) && (ci == 0))
    new_nodval = 0.0;
  if((ri == 3) && (ci == 1))
    new_nodval = 0.0;
  if((ri == 3) && (ci == 2))
    new_nodval = 0.0;
  if((ri == 3) && (ci == 3))
    new_nodval = 0.0;


  return (new_nodval);
}


/**
   function computes flux (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param nodval     - prescribed nodal value
   @param ri         - row index
   @param ci         - column index
   @param nn         - number of node
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
double saltmat4::transmission_flux(double nodval,long ri,long ci,long nn,long bc,long ipp)
{
  long k;
  double flux,x1,x2,x3,x4;
  flux = 0.0;
  
  k=Gtt->give_dof(nn,0);
  if (k>0)   {x1 = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {x1 = 0.0;}
  if (k<0)   {x1 = Tb->lc[0].pv[0-k-1].getval();}
  k=Gtt->give_dof(nn,1);
  if (k>0)   {x2 = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {x2 = 0.0;}
  if (k<0)   {x2 = Tb->lc[0].pv[0-k-1].getval();}
  k=Gtt->give_dof(nn,2);
  if (k>0)   {x3 = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {x3 = 0.0;}
  if (k<0)   {x3 = Tb->lc[0].pv[0-k-1].getval();}
  k=Gtt->give_dof(nn,3);
  if (k>0)   {x4 = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {x4 = 0.0;}
  if (k<0)   {x4 = Tb->lc[0].pv[0-k-1].getval();}
  
  if((ri == 0) && (ci == 0))
    flux = get_transmission_flux_11(nodval,x1,x2,x3,x4,bc,ipp);
  if((ri == 0) && (ci == 1))
    flux = 0.0;
  if((ri == 0) && (ci == 2))
    flux = 0.0;
 if((ri == 0) && (ci == 3))
    flux = 0.0;
  
  if((ri == 1) && (ci == 0))
    flux = 0.0;
  if((ri == 1) && (ci == 1))
    flux = 0.0;
  if((ri == 1) && (ci == 2))
    flux = 0.0;
  if((ri == 1) && (ci == 3))
    flux = 0.0;
  

  if((ri == 2) && (ci == 0))
    flux = 0.0;
  if((ri == 2) && (ci == 1))
    flux = 0.0;
  if((ri == 2) && (ci == 2))
    flux = 0.0;
  if((ri == 2) && (ci == 3))
    flux = 0.0;


  if((ri == 3) && (ci == 0))
    flux = 0.0;
  if((ri == 3) && (ci == 1))
    flux = 0.0;
  if((ri == 3) && (ci == 2))
    flux = 0.0;
  if((ri == 3) && (ci == 3))
    flux = 0.0;

  return (flux);
}


/**
   function creates correct new nodal value on the boundary (transmission) for 1st medium
   @param new_nodval - new prescribed value near the boundary
   @param bv         - value of prescribed value near the boundary
   @param x1 ... x3  - actual unknowns on the boundary
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
double saltmat4::get_transmission_nodval_11(double bv,double x1,double x2,double x3,double x4,long bc,long ipp)
{
  double nodval;

  switch (bc){//type of prescribed variable
  case 30:{//transmission
    nodval = bv;//should be changed
    break;
  } 
  default:{
    fprintf (stderr,"\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__);
    exit(0);
  }
  }
  
  return(nodval);
}


/**
   function creates correct transfer coefficient on the boundary (transmission) for 1st medium
   @param f11        - correct transfer coefficient
   @param x1 ... x3  - actual unknowns on the boundary
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
double saltmat4::get_transmission_transcoeff_11(double x1,double x2,double x3,double x4,long bc,long ipp)
{
  double trc;

  switch (bc){//type of prescribed variable
  case 30:{//transmission
    trc=1.0;//should be changed
    break;
  }
  default:{
    fprintf (stderr,"\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__);
    exit(0);
  }
  }

  return(trc);
}


/**
   function creates flux on the boundary (transmission - convective mass transfer) for 1st medium
   @param new_nodval - flux on the boundary
   @param bv         - prescribed value near the boundary
   @param x1 ... x3  - actual unknowns on the boundary
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
double saltmat4::get_transmission_flux_11(double bv,double x1,double x2,double x3,double x4,long bc,long ipp)
{
  double flux;

  switch (bc){//type of prescribed variable
  case 30:{//transmission - boundary flux
    flux = (bv - x1);//should be changed
    break;
  }
  default:{
    fprintf (stderr,"\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__);
    exit(0);
  }
  }
  
  return(flux);
}

/**
   function computes all variables in nodes
   @param compother  - number of other components
   @param ipp        - first integration point on element
   @param x1 ... x3  - actual unknowns on the boundary
*/

double saltmat4::get_othervalue(long compother,long ipp, double x1,double x2,double x3,double x4)
{
  double other, b,a3,cb;
  int kod;
  
  switch (compother){
  case 1:{
	     CorD(15,kod,1000,0,x2,cb,b,a3);
		 other= x1*x2+cb+fabs(x3);
		 }
	  break;
  case 2:other  = Tm->ip[ipp].eqother[9];
	  break;
  case 3:other  = Tm->ip[ipp].eqother[10];
	  break;
  case 4:other  = Tm->ip[ipp].eqother[11];
	  break;
  case 5: other  = get_rel_hum2 (x1); // Tm->ip[ipp].eqother[5];
	  break;
  case 6:other  = 0.0; //Tm->ip[ipp].eqother[5];
	  break;
  case 7: CorD(15,kod,1000,0,x2,other,b,a3);
	  break;
  case 0:{//first unknown

	  // nefunguje.. ja to mam v uzlech vypocitane, ale tady to jde pres IPP.....
/*	  if(ipp >11) ipp = 10;
    if( Tt->nodes[ipp].eqother[9] == x1)
	{
		other = Tt->nodes[ipp].eqother[8];
	}
	else
	{
		other = -10000.0;
	}*/
	  CorD(15,kod,1000,0,x2,cb,b,a3);
	  other= x1*x2+cb+x3;///pozor..nevim, zda to tam ma byt...
	  //other = 0.0;
      break; 
  }
  default:{
    fprintf (stderr,"\n\n unknown type of component is required in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return (other);
}

/**
     function prints names of all variables in nodes
     @param out       - output file
     @param compother - number of other components
*/
void saltmat4::print_othervalue_name(FILE *out,long compother)
{
  switch (compother){
  case 0:{//first unknown
    fprintf (out,"First unknown (Units)        ");//should be cha4nged
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown type of component is required in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}
/** tlak nasycenych vodnich par
    @param t  - temperature K
*/

double  saltmat4::pgws(double x1, double x2, double x3, double x4)
{
    double pgw;

	pgw = exp(23.5771 - 4042.9/(x4 - 37.58));

	return (pgw);
}
 /** Permeabilita Vodni Pary 
    @param t  - temperature K
    @param p  -  presure Pa

*/
double saltmat4::permeabilitavodnipary(double x1, double x2, double x3, double x4)
{

	double Da, Dp,rv,pa, mi, p;
	
	CorD(4,kd,1,0,x1,mi,a2,a3);

	rv = 461.5;
	pa = 101325;
	p = 101325;

	Da = (2.306e-5 * pa)/(rv * x4 * p)*pow((x4/294.15),1.81);
	Dp = Da/mi;
	return (Dp);

 }

void saltmat4::CorD(int cislochar, int &kvyhl,double in,int rhw, double x, double & y, double & z, double &z2)
{
int i;
  switch (MatChar[cislochar])
     {
     case 0:
     break;
     case 1: y = MatConst[cislochar];
     break;
     case 2:   {
                int i = 1;

                int psl, prad;
                prad  = int (MatData[cislochar][0][0]);
                psl = int (MatData[cislochar][1][0]);
                if(rhw == 1)
                        {
                        if (x <=0)
                                {
                                x = 0;
                                y = MatData[cislochar][0][1];
                                if (psl == 3)
                                        {
                                        z = MatData[cislochar][2][1];
                                        }

                                }
                        else {
                                if (x > in)
                                        {
                                        x = in;
                                        y = MatData[cislochar][0][prad];
                                        if (psl == 3)
                                                {
                                                z = MatData[cislochar][2][prad];
                                                }
                                        }
                                else
                                        {
                                        while (x > MatData[cislochar][1][i]){
                                        i++;
                                        }

                                        y = MatData[cislochar][0][i-1] + ((x - MatData[cislochar][1][i-1])*(MatData[cislochar][0][i] - MatData[cislochar][0][i-1]))/(MatData[cislochar][1][i] - MatData[cislochar][1][i-1]);
                                        if (psl == 3)
                                                {
                                                z = MatData[cislochar][2][i-1] + ((x - MatData[cislochar][1][i-1])*(MatData[cislochar][2][i] - MatData[cislochar][2][i-1]))/(MatData[cislochar][1][i] - MatData[cislochar][1][i-1]);
                                                }
                                        }
                                }
                        }
                else    {
                                if (x <=0)
                                {
                                x = 0;
                                y = MatData[cislochar][1][1];
                                if (psl == 3)
                                        {
                                        z = MatData[cislochar][2][1];
                                        }

                                }
                                else {
                                if (x > in)
                                        {
                                        x = in;
                                        y = MatData[cislochar][1][prad];
                                        if (psl == 3)
                                                {
                                                z = MatData[cislochar][2][prad];
                                                }
                                        }
                                else
                                        {
                                        while (x > MatData[cislochar][0][i]){
                                        i++;
                                        }

                                        y = MatData[cislochar][1][i-1] + ((x - MatData[cislochar][0][i-1])*(MatData[cislochar][1][i] - MatData[cislochar][1][i-1]))/(MatData[cislochar][0][i] - MatData[cislochar][0][i-1]);
                                        if (psl == 3)
                                                {
                                                z = MatData[cislochar][2][i-1] + ((x - MatData[cislochar][0][i-1])*(MatData[cislochar][2][i] - MatData[cislochar][2][i-1]))/(MatData[cislochar][0][i] - MatData[cislochar][0][i-1]);
                                                }
                                        }
                                }
                }
                kvyhl = 2;
                }
     break;
     case 30:  y = MatFunce[cislochar][0];
               z = MatFunce[cislochar][1];
               kvyhl = 30;
			   break;
     case 31:  y = MatFunce[cislochar][0];
               z = MatFunce[cislochar][1];
               if (cislochar == 6)
			   {
				   z2 = MatFunce[cislochar][2];
			   }
               kvyhl = 31;
			   break;
     case 32:  y = MatFunce[cislochar][0];
               z = MatFunce[cislochar][1];
               kvyhl = 32;
			   break;
     case 33: //zatim nic
		 break;
	 case 40:{
		 int pradA, pradD;
		 pradA  = int(MatData[cislochar][0][0]);
		 pradD  = int(MatData[cislochar][2][0]);
		 if (x <=0)
		 {
			 x = 0;
			 y = MatData[cislochar][1][1];
			 z = MatData[cislochar][3][1];
		 }
		 else
		 {
			 if (x > in)
			 {
				 y = MatData[cislochar][1][pradA];
				 z = MatData[cislochar][3][pradD];
			 }
			 else
			 {
				 i = 1;
				 while (x > MatData[cislochar][0][i]){
					 i++;
				 }
				 y = MatData[cislochar][1][i-1] + ((x - MatData[cislochar][0][i-1])*(MatData[cislochar][1][i] - MatData[cislochar][1][i-1]))/(MatData[cislochar][0][i] - MatData[cislochar][0][i-1]); 
				 i = 1;
				 while (x > MatData[cislochar][2][i]){
					 i++;
				 }
				 z = MatData[cislochar][3][i-1] + ((x - MatData[cislochar][2][i-1])*(MatData[cislochar][3][i] - MatData[cislochar][3][i-1]))/(MatData[cislochar][2][i] - MatData[cislochar][2][i-1]);
			 }
		 }
		 kvyhl = 40;
			 }
		 break;
  }

}

void saltmat4::binding_izoterm_derivation(double x2, double & derbi)
{
  int i = 1;
  if ( x2 < 0)
  {
      x2 = 0;
  }
  
  while (x2 > MatData[15][0][i]){
	  i++;
  }
  
  derbi = (MatData [15][1][i] - MatData [15][1][i-1])/(MatData [15][0][i] - MatData [15][0][i-1]);
  //TODO: Add your source code here
}

void saltmat4::kapa_values (int kod, long ipp,double x1, double xpv, double ineq1, double &kapa)
{
  double kapa1, kapa2, kapa3;
  double smc,a,n;
  CorD(7,kod,1,0,x1,smc,a,n);

  if( x1 > smc) x1 = smc;

  switch (kod){
  case 0: kapa = 0.0;
	  break;
  case 1: CorD(5,kod,1,0,x1,kapa,kapa1,kapa2);
	  break;
  case 2: CorD(5,kod,1,0,x1,kapa,kapa1,kapa2);
	  break;
  case 30:{
	  CorD(5,kod,1,0,x1,kapa1,kapa2,kapa3);
	  kapa = kapa1*exp(kapa2*x1);
	  break;
		  }
  case 40: hystereze2 (5,x1, xpv, ineq1, kapa,kapa1, ipp,Init[5]);
	   //fprintf (Outt,"\n Time = %lf x2 = %lf  xpv = %lf  ineq1 = %lf  kapa = %3.15e IPP = %ld \n",Tp->time ,x1,xpv,ineq1,kapa, ipp);
	  break;
  }

}

void saltmat4::salt_diffusivity_values (int kod, long ipp, double x2, double xpv,double ineq1, double &diff)
{
  double diff1, diff2, diff3;

  
  switch (kod){
  case 0: diff = 0.0;
	  break;
  case 1: CorD(14,kod,1000,0,x2,diff,diff1,diff2);
	  break;
  case 2: CorD(14,kod,1000,0,x2,diff,diff1,diff2);
	  break;
  case 30:{
	  CorD(14,kod,1000,0,x2,diff1,diff2,diff3);
	  diff = diff1*exp(diff2*x2);
	  break;
		  }
  case 40: hystereze2 (14,x2,xpv,ineq1,diff,diff1,ipp,Init[14]);
//	 fprintf (Outt,"\n Time = %lf x2 = %lf  xpv = %lf  ineq1 = %lf  diff = %lf IPP = %ld \n",Tp->time ,x2,xpv,ineq1,diff, ipp);

	  break;
  }

}

void saltmat4::binding_izoterm_derivation_values (int kod, long ipp, double x2, double xpv, double ineq1, double &cb, double &dcbdcf)
{
  double a,b,a3,Cba;
  
  if (x2 <= 0) x2 = 1e-10;
 
  switch (kod)
     {
  case 0: dcbdcf = 0.0;
	  cb = 0.0;
	  break;
  case 2: //binding_izoterm_derivation(x2,dcbdcf);
	  dcbdcf =derivation_dy_dx(15,x2,0,1);
	  CorD(15,kod,1000,0,x2,cb,b,a3);
	  break;
  case 30:{
	  CorD(15,kod,1000,0,x2,a,b,a3);
	  Cba =  1/(1/(a*b*x2)+1/b);
	  dcbdcf = Cba*Cba/(a*b*x2*x2);
	  cb = Cba;
	  break;
		  }
  case 31:{
	  CorD(15,kod,1000,0,x2,a,b,a3);
	  dcbdcf = b*a*pow(x2,(b-1));
	  cb = 0.0;
	  break;
		  }
  case 40: hystereze2 (15,x2,xpv,ineq1,cb,dcbdcf, ipp,Init[15]);
	  break;
  }

 
}

/**
   function selects auxiliary values
   
   JK, 7.1.2008
*/
void saltmat4::give_values (long ipp,double *av, double *pv, double *eq)
{
  av[0] = Tm->ip[ipp].av[0];
  av[1] = Tm->ip[ipp].av[1];
  av[2] = Tm->ip[ipp].av[2];
  av[3] = Tm->ip[ipp].av[3];

  pv[0] = Tm->ip[ipp].pv[0];
  pv[1] = Tm->ip[ipp].pv[1];
  pv[2] = Tm->ip[ipp].pv[2];
  pv[3] = Tm->ip[ipp].pv[3];


  eq[0] = Tm->ip[ipp].eqother[0];
  eq[1] = Tm->ip[ipp].eqother[1];
  eq[2] = Tm->ip[ipp].eqother[2];
  eq[3] = Tm->ip[ipp].eqother[3];
  eq[4] = Tm->ip[ipp].eqother[4];
  eq[5] = Tm->ip[ipp].eqother[5];
  eq[6] = Tm->ip[ipp].eqother[6];
  eq[7] = Tm->ip[ipp].eqother[7];

}

/**
   function computes auxiliary values which are necessary for future computation
   
   JM, 29.5.2007
*/
void saltmat4::aux_values (long ipp,double *in,double *inp, double *ineq,double *out)
{
  double x1,x2,x3,x4,x1p, x2p, x3p,x4p, ypv1,ypv2,ypv3,ypv4,ypv5,ypv6,ypv7,ypv0;
 double a1,a2,cfmax,jm1, d,cb, dcbdcf,fi,dfdw;
  
  x1 = in[0];
  x2 = in[1];
  x3 = in[2];
  x4 = in[3];

  x1p = inp[0];
  x2p = inp[1];
  x3p = inp[2];
  x4p = inp[3];
  

  ypv0 =ineq[0];
  ypv1 =ineq[1];
  ypv2 =ineq[2];
  ypv3 =ineq[3];
  ypv4 =ineq[4];
  ypv5 =ineq[5];
  ypv6 =ineq[6];
  ypv7 =ineq[7];


  	
	
	kapa_values(MatChar[5],ipp,x1,x1p,ypv0, jm1); 
	out[0]=jm1;
  
	sorption_izoterms_values(MatChar[6],ipp,x1,x1p,ypv1,fi,dfdw); 
	out[1]=fi;
	out[2]=dfdw;

	salt_diffusivity_values(MatChar[14],ipp,x2,x2p,ypv3,d);
	out[3]=d;

  	binding_izoterm_derivation_values(MatChar[15],ipp,x2,x2p,ypv4,cb,dcbdcf);
    out[4]=cb;
	out[5]=dcbdcf;
  
	
	CorD(16,kd,0,0,x1,cfmax,a1,a2);
	out[6]=cfmax;

	
double smc;
	CorD(7,kd,1,0,x1,smc,a1,a2);

	out[7]=smc;

	
	/*	if(ipp ==0)
		{
			fprintf(Outt1, "%le  %le  %le  %le  %le  %le  %le\n",  Tp->time, x1, jm1, fi,x2, d, cb);// kapa a decko
		}

		if(ipp ==15)
		{
			fprintf(Outt2, "%le  %le  %le  %le  %le  %le  %le\n",  Tp->time, x1, jm1, fi,x2, d, cb);// kapa a decko
		}*/
  
	if (Tp->time > 0 && Tp->time < 1*365*86400)
	{
		/*if(ipp ==24)
		{
			fprintf(Outt2, "%le  %le  %le  %le  %le  %le  %le\n",  Tp->time, x1, jm1, fi,x2, d, cb);// kapa a decko
		}*/
	/*	if(ipp ==0)
		{
			fprintf(Outt2, "cas= %le  w= %le cf= %le  cb= %le cc= %le Ct= %le\n",  Tp->time/86400, x1, x2, cb,x3, x1*x2+cb+x3);// kapa a decko
		}*/
	}
}



/**
   function saves auxiliary values
   
   JK, 7.1.2008
*/
void saltmat4::save_values (long ipp,double *out)
{
  Tm->ip[ipp].eqother[0]=out[0];
  Tm->ip[ipp].eqother[1]=out[1];
  Tm->ip[ipp].eqother[2]=out[2];
  Tm->ip[ipp].eqother[3]=out[3];
  Tm->ip[ipp].eqother[4]=out[4];
  Tm->ip[ipp].eqother[5]=out[5];
  Tm->ip[ipp].eqother[6]=out[6];
  Tm->ip[ipp].eqother[7]=out[7];

}

/**
   function defines inital values of quantities at integration points
   
   @param ipp - integration point pointer
   @param ido - index in array eq_other   

   JM, 29.5.2007
*/
void saltmat4::initvalues (long ipp,long ido)
{

	double x1,x2,x3,x4;

	x1 = Tm->ip[ipp].av[0];
	x2 = Tm->ip[ipp].av[1];
	x3 = Tm->ip[ipp].av[2];
	x4 = Tm->ip[ipp].av[3];
}


void saltmat4::der_value_hyst (int matchar,int kod, double pv, double & outvalue,double & outvalue2, long ipp)
{

	switch (kod){
	case 0:
		break;
	case 1:{
		break;
		   }
	case 2:{
		break;
		   }
	case 30:{
		break;
			}
	case 31:{
		break;
			}
	case 40:{

		outvalue = derivation_dy_dx(matchar,pv,0,1);
		outvalue2 = derivation_dy_dx(matchar,pv,2,3);

		}
		break;
	}

}

double saltmat4::derivation_saturation_water_vapour_pressure_temperature(double x1, double x2, double x3, double x4)
{
  return ((4042.9 * exp(23.5771-4042.9/(x4 - 37.58)))/((x4-37.58)*(x4-37.58)));   //TODO: Add your source code here
}

double saltmat4::latent_heat_of_evaporation_of_water(double x1, double x2, double x3, double x4)
{
        return (2.5008e6)*pow((273.15/x4),(0.167+x4*3.67e-4));
}

void saltmat4::sorption_izotherm_derivation(double x1, double x2,  double x3, double x4, double & derfi)
{
 int i = 1;
if ( x1 < 0)
     {
     x1 = 0;
      }

 while (x1 > MatData[6][1][i]){
                              i++;
                              }
 derfi = (MatData [6][0][i] - MatData [6][0][i-1])/(MatData [6][1][i] - MatData [6][1][i-1]);
    
}

void saltmat4::sorption_izoterms_values(int kod, long ipp, double x1,double xpv, double ineq1, double & fi, double & dfdw)
{
 /*
 kod ..... 0- jen vlhkost, 1 - jen derivaci...
 */
 int s;
 double w_hyg,rh_hyg, smc, hvezda;
// double vyh1, vyh2, vyh3;
 double x2, x3;

 x2 = 0.0;
 x3 = 0.0;
       
       if (x1 < 0) x1 = 0;

       switch (kod)
          {
          case 2:   CorD(6,s,1,0,x1,fi,a2,a3);
					dfdw = derivation_dy_dx(6,x1,0,1);
          break;
          case 30:  //CorD(6,s,1,x1,a1,hmrh,a3);
                      CorD(6,s,0,0,x1,w_hyg,rh_hyg,a3);
                      hvezda =sortpion_isotherm_root_shifted(x1,w_hyg ,rh_hyg);
                      CorD(7,s,0,0,x1,smc,a2,a3);
					  give_data_si_root_fi(x1,x2,x3,w_hyg, smc, rh_hyg,hvezda,fi);
					  give_data_si_root_dfidw(x1,x2,x3,rh_hyg,smc,w_hyg,hvezda,dfdw);
                    
          break;
          case 31: 
			  fi  = a1*pow((1-(log(x1))/a2),(-1/a3));
			  dfdw = (a1/(a2*x1*a3))*pow((1-(log(x1))/a2),(-1-(1/a3)));
		  break ; 
		  case 40:{
			   hystereze2 (6,x1, xpv, ineq1,fi,dfdw, ipp,Init[6]);
			   	
				  }
			  break;
          }
}
 

void saltmat4::give_data_si_root_fi(double x1, double x2, double x3, double w_hyg, double w_sat, double rh_hyg, double shift_w,  double & relh)
{

  if (x1 < w_hyg)
     {
     relh = 1 - (1-x1/shift_w)*(1-x1/shift_w);
     }
  else
     {
     if (x1 > w_sat)
          {
          relh = 1;
          }
     else
          {
          relh = rh_hyg+(1-rh_hyg)*(x1-w_hyg)/(w_sat-w_hyg);
          }
     }

 // moissat = Smc ;
    //TODO: Add your source code here
} 

double saltmat4::get_rel_hum(double w)
{
//double saltmat4::water_content_relhum (long nid,double in,double inp, double ineq)
	double relhum;
		
	//water_content_relhum(MatChar[5],nid,in,inp,ineq,relhum,derivace); 
    
	return (relhum);
}

double saltmat4::sortpion_isotherm_root_shifted(double x1, double w_hyg, double rh_hyg)
{
  return(w_hyg/(1-sqrt(1-rh_hyg)));
}

void saltmat4::give_data_si_root_dfidw(double x1, double x2, double x3, double rh_hyg, double w_sat, double w_hyg, double shift_w, double & dfdw)
{

     if ( x1< 0) x1 = 0;
     if (x1>w_sat) x1 = w_sat;

     if(x1<w_hyg)
          {
          dfdw = 2*(shift_w-x1)/(shift_w*shift_w);
          }
     else
          {
          dfdw = (1-rh_hyg)/(w_sat-w_hyg);
          }
}

//double saltmat4::get_moisture(double rh)

void saltmat4::get_moisture (long nid,double in,double *inp, double &out)
{
  int s;
 double moist,hmrh, mhmc, smc, u, a, n, rh;
 double t;

  t = Tp->time;
        rh = in;
       if (rh < 0) rh = 0;

       s = MatChar[6];      // zjisteni zpusobu zadani SI ... 2-data, 30-ROOT, 31-HANSEN

       switch (s)
          {
          case 2:   CorD(6,s,1,1,rh,moist,a1,a3);
		  break;
          case 30:  CorD(6,s,1,0,rh,a1,hmrh,a3);
                    CorD(7,s,0,0,rh,smc,a2,a3);
                    CorD(6,s,0,0,rh,mhmc,hmrh,a3);
                    kun2->give_data(rh,mhmc, smc, hmrh,moist);
                    //kunmat->give_data(fi,mhmc, smc, hmrh,moist);
          break;
          case 31:  CorD(6,s,0,0,0,u,a,n);
                    moist = kun2->si_kk_hansen(rh, 0,u,a,n);
          break ;
		  case 40: {
					if (t < Init[6])
					{
						int i = 1;
						while (rh > MatData[6][1][i]){
							i++;
						}
						//  moist = MatData[6][1][i-1] + ((rh - MatData[6][0][i-1])*(MatData[6][1][i] - MatData[6][1][i-1]))/(MatData[6][0][i] - MatData[6][0][i-1]); 
						moist = MatData[6][0][i-1] + ((rh - MatData[6][1][i-1])*(MatData[6][0][i] - MatData[6][0][i-1]))/(MatData[6][1][i] - MatData[6][1][i-1]); 
					
					}
					else
					{
					moist = inverse_hystereze_sorption_izoterms (nid, in,inp[11],inp[12]);
					}
				   }
			  break;
	   }
	   out = moist;

    //TODO: Add your source code here
} 


double saltmat4::get_moisture2 ( double rh)
{
  int s;
 double moist,hmrh, mhmc, smc, u, a, n;


     if (rh < 0) rh = 0;

       s = MatChar[6];      // zjisteni zpusobu zadani SI ... 2-data, 30-ROOT, 31-HANSEN

       switch (s)
          {
          case 2:   CorD(6,s,1,1,rh,moist,a1,a3);
		  break;
          case 30:  CorD(6,s,1,0,rh,a1,hmrh,a3);
                    CorD(7,s,0,0,rh,smc,a2,a3);
                    CorD(6,s,0,0,rh,mhmc,hmrh,a3);
                    kun2->give_data(rh,mhmc, smc, hmrh,moist);
                    //kunmat->give_data(fi,mhmc, smc, hmrh,moist);
          break;
          case 31:  CorD(6,s,0,0,0,u,a,n);
                    moist = kun2->si_kk_hansen(rh, 0,u,a,n);
          break ;
	   }
	   return( moist);

    //TODO: Add your source code here
} 
long saltmat4::cycle_detection (double *r,double *pr, double *ppr)
{
	double t,pvt, ppvt;

	t = r[3];
	pvt = pr[3];
	ppvt = ppr[3];

	if (ppvt > 273.15 && pvt > t && t < 273.15 && ppvt > pvt && pvt < 273.15)
	{
		//fprintf(Outt, "\nTime je %lf  t je %lf a pvt je %lf a ppvt je %lf",Tp->time,t, pvt, ppvt);
		//printf ("\nTime je %lf  t je %lf a pvt je %lf a ppvt je %lf",Tp->time,t, pvt, ppvt);
		return(1);
	}
	else {
		return(0);
	}

}

double saltmat4::derivation_dy_dx (int matchar, double prom, int pomk1, int pomk2)

{
	int i = 1;
	double outvalue;
	
	while (prom > MatData[matchar][pomk1][i]){
			i++;
		}
	outvalue = (MatData [matchar][pomk2][i] - MatData [matchar][pomk2][i-1])/(MatData [matchar][pomk1][i] - MatData [matchar][pomk1][i-1]);
	return(outvalue);

}

void saltmat4::hystereze2 (int matchar, double x, double xpv, double ineq1,double & outvalue,double & outvalue2, long ipp, long timeH)
{
  double xav,yav, newder,ypv,yavA,yavD, ypvA, ypvD, derpvA,derpvD, other1,xpvD, xpvA;
  int hyst, other;
  double ztracene;
  //double  k1,k2, k3, k4, k5, k6;
  double t;
  t = Tp->time;
 
  /* x ... actual value
     xpv  ... previus value
	 ineq1,ineq2 ... values of eqother

  */
  newder = 0.0;
  if (t < timeH)
  {
	  switch (matchar)
		  {
		  case 5: {
			  // pocatek navlhani
			//  CorD(5,other,1,0,x,outvalue,outvalue2,other1);
			  // pocatek vysouseni
			  CorD(5,other,1,0,x,ztracene,outvalue,other1);
			   	  }
			  break;
		  case 6: {
			  // pocatek navlhani
			 // CorD(6,other,1,0,x,outvalue,ztracene,other1);
			  // pocatek vysouseni
			  CorD(6,other,1,0,x,ztracene,outvalue,other1);
			  outvalue2 = derivation_dy_dx(6,x,2,3);
			  	  }
			  break;
		  case 14: {
			  // pocatek navlhani
			   //CorD(14,other,1000,0,x,outvalue,ztracene,other1);
			  // pocatek vysouseni
			  CorD(14,other,1000,0,x,ztracene,outvalue,other1);
			  	   }
			  break;
		  case 15:{
			  // pocatek navlhani
			  //outvalue2 =derivation_dy_dx(15,x,0,1);
			  //CorD(15,other,1000,0,x,yavA,ztracene,other1);
			  // pocatek vysouseni
			  outvalue2 =derivation_dy_dx(15,x,2,3);
			  CorD(15,other,1000,0,x,ztracene,yavA,other1);
			  outvalue = yavA;
			   	  }
			  break;
		  }
	  
  }
  else
  {
	  switch(matchar)
	  {
	  case 5:{
		  xav = x;
		  ypv = ineq1;
		  CorD(5,other,1,0,x,yavA,yavD,other1);
		  CorD(5,other,1,0,xpv,ypvA,ypvD,other1);
		  
		  der_value_hyst(matchar,40,xpv,derpvA,derpvD,ipp); 
		  if(xav > xpv)            
				{
				  hyst = 0;
				}
		  else	{
				  hyst = 1;
				}
		  switch(hyst)
			{
				case 0:{			// zvysovani promene....
						if(ypv == ypvA)                    
							{
							yav = yavA;
							}
						else {
							 newder = (k1[matchar]*derpvD*(ypv-ypvA)*(ypv-ypvA)+k2[matchar]*derpvA*(ypv -ypvD)*(ypv - ypvD))/((ypvD-ypvA)*(ypvD-ypvA));
							 yav = (xav-xpv)*newder*k3[matchar]  + ypv;
							 if(yav > yavA) yav = yavA;
							 if(yav <yavD) yav = yavD;
							}
						break;
					   }
				case 1:{			// klesani zakl promene
						if(ypv == ypvD)
							{
							yav = yavD;
							}
						else {
							newder  = (k4[matchar]*derpvD*(ypv-ypvA)*(ypv-ypvA)+k5[matchar]*derpvA*(ypv -ypvD)*(ypv - ypvD))/((ypvD-ypvA)*(ypvD-ypvA));
							yav = (xav-xpv)*newder*k6[matchar] + ypv;
							if(yav > yavA) yav = yavA;
							if(yav < yavD)yav = yavD;
							}
						break;
					   }
		  }
		  //if (ipp == 10)
			//  fprintf (Outt,"\n xav = %2.15e  xpv = %2.15e  yav = %2.15e  ypv = %2.15e newder - %2.15e  IPP = %ld Time = %lf \n",xav,xpv,yav,ypv,newder, ipp,Tp->time );
	
		  outvalue = yav;
		  outvalue2 = -10000.0;
		  break;
		  }
	  case 6:
		  {
			  xav = x;
			  ypv = ineq1;		  
			  CorD(6,other,1,0,xav,yavA,yavD,other1);
			  
			  int pradA, pradD, i;
			  
			  pradA  = int(MatData[6][0][0]);
			  pradD  = int(MatData[6][2][0]);
			  if (ypv <=0)
			  {
				  ypv = 0;
				  xpvA = MatData[6][0][1];
				  xpvD = MatData[6][2][1];
			  }
			  else
			  {
				  if (ypv > 1)
				  {
					  xpvA = MatData[6][0][pradA];
					  xpvD = MatData[6][2][pradD];
				  }
				  else
				  {
					  i = 1;
					  while (ypv > MatData[6][1][i]){
						  i++;
					  }
					  xpvA = MatData[6][0][i-1] + ((ypv - MatData[6][1][i-1])*(MatData[6][0][i] - MatData[6][0][i-1]))/(MatData[6][1][i] - MatData[6][1][i-1]); 
					  i = 1;
					  while (ypv > MatData[6][3][i]){
						  i++;
					  }
					  xpvD = MatData[6][2][i-1] + ((ypv - MatData[6][3][i-1])*(MatData[6][2][i] - MatData[6][2][i-1]))/(MatData[6][3][i] - MatData[6][3][i-1]);
				  }
			  }
			  derpvA = derivation_dy_dx(6,xpvA,0,1);
		      derpvD = derivation_dy_dx(6,xpvD,2,3);
			  if(xav > xpv)
			  {
				  hyst = 0;
			  }
			  else{
				  hyst = 1;
			  }
			  
			  switch(hyst)
			  {
				case 0:{			// zvysovani promene....
						if(fabs(xpv-xpvA) < 1e-8)                    
							{
							yav = yavA;
							}
						else {
							 newder = (k1[matchar]*derpvD*(xpv-xpvA)*(xpv-xpvA)+k2[matchar]*derpvA*(xpv -xpvD)*(xpv - xpvD))/((xpvD-xpvA)*(xpvD-xpvA));
							 yav = (xav-xpv)*newder*k3[matchar]  + ypv;
							 if(yav > yavA) yav = yavA;
							 if(yav < yavD) yav = yavD;

							}
						break;
					   }
				case 1:{			// klesani zakl promene
						if(fabs(xpv-xpvD) < 1e-8) 
							{
							yav = yavD;
							}
						else {
							newder  = (k4[matchar]*derpvD*(xpv-xpvA)*(xpv-xpvA)+k5[matchar]*derpvA*(xpv -xpvD)*(xpv - xpvD))/((xpvD-xpvA)*(xpvD-xpvA));
							yav = (xav-xpv)*newder*k6[matchar] + ypv;
							if(yav > yavA) yav = yavA;
							if(yav < yavD) yav = yavD;

							}
						break;
					   }
			  }
			  
			  if (yav == yavA || yav == yavD)
				  {
					if ((xav-xpv)==0)
					{
					  newder = (yav-ypv)/(1e-8);
					}
					else
					{
					  newder = (yav-ypv)/(xav - xpv);
					}
					//  if (newder== 0) newder = 1e-5;
					  //fprintf (Outt,"\n xav = %lf  xpv = %lf  yav = %lf  ypv = %lf newder - %lf  IPP = %ld Time = %lf \n",xav,xpv,yav,ypv,newder, ipp,Tp->time );
					
				  }
			  else
				{
				  newder = (yav - ypv)/(xav-xpv);
			  }
			  if (newder== 0)
			  {
				  newder = 1e-5;
			  }
			  /*	if(ipp ==8)
					{
					fprintf(Outt, "%le  %le  %le  %le  %le  %le  %le  %le\n",  Tp->time, xav, xpv, yav,ypv, yavA, yavD, newder);// kapa a decko
					}*/
			  if (fabs(yav-ypv)<1e-8) newder = 1e-5;
			  outvalue = yav;
			  outvalue2 = newder;
			  			  
		  }
		  break;
	  case 14:
		  {
			  xav = x;
			  ypv = ineq1;
			  CorD(14,other,1000,0,xav,yavD,yavA,other1);
			  CorD(14,other,1000,0,xpv,ypvD,ypvA,other1);
			  der_value_hyst(matchar,40,xpv,derpvD,derpvA,ipp);
			  if(xav > xpv)            
				{
				  hyst = 0;
			  }
			  else	{
				  hyst = 1;
			  }
			  switch(hyst)
			  {
				case 0:{			// zvysovani promene....
						if(ypv == ypvA)                    
							{
							yav = yavA;
							}
						else {
							 newder = (k1[matchar]*derpvD*(ypv-ypvA)*(ypv-ypvA)+k2[matchar]*derpvA*(ypv -ypvD)*(ypv - ypvD))/((ypvD-ypvA)*(ypvD-ypvA));
							 yav = (xav-xpv)*newder*k3[matchar]  + ypv;
							 if(yav < yavA) yav = yavA;
							 if(yav > yavD) yav = yavD;
							}
						break;
					   }
				case 1:{			// klesani zakl promene
						if(ypv == ypvD)
							{
							yav = yavD;
							}
						else {
							newder  = (k4[matchar]*derpvD*(ypv-ypvA)*(ypv-ypvA)+k5[matchar]*derpvA*(ypv -ypvD)*(ypv - ypvD))/((ypvD-ypvA)*(ypvD-ypvA));
							yav = (xav-xpv)*newder*k6[matchar] + ypv;
							if(yav > yavD)yav = yavD;
							if(yav < yavA)yav = yavA;
							}
						break;
					   }
			  }
			  outvalue = yav;
			  outvalue2 = -10000.0;
		  }
		  break;
	  case 15:
		  {
			  xav = x;
			  ypv = ineq1;
			  CorD(15,other,1000,0,xav,yavA,yavD,other1);
			  CorD(15,other,1000,0,xpv,ypvA,ypvD,other1);
			  der_value_hyst(matchar,40,xpv,derpvD,derpvA,ipp);
			  
			  if(xav > xpv)            
				{
				  hyst = 0;
			  }
			  else	{
				  hyst = 1;
			  }
			  switch(hyst)
			  {
				case 0:{			// zvysovani promene....
						if(ypv == ypvD)                    
							{
							yav = yavD;
							}
						else {
							 newder = (k1[matchar]*derpvD*(ypv-ypvA)*(ypv-ypvA)+k2[matchar]*derpvA*(ypv -ypvD)*(ypv - ypvD))/((ypvD-ypvA)*(ypvD-ypvA));
							 yav = (xav-xpv)*newder*k3[matchar]  + ypv;
							 if(yav > yavA)yav = yavA;
							 if(yav < yavD) yav = yavD;
							}
						break;
					   }
				case 1:{			// klesani zakl promene
						if(ypv == ypvA)
							{
							yav = yavA;
							}
						else {
							newder  = (k4[matchar]*derpvD*(ypv-ypvA)*(ypv-ypvA)+k5[matchar]*derpvA*(ypv -ypvD)*(ypv - ypvD))/((ypvD-ypvA)*(ypvD-ypvA));
							yav = (xav-xpv)*newder*k6[matchar] + ypv;
							if(yav < yavA)yav = yavA;
							if(yav > yavD) yav = yavD;
							}
						break;
					   }
			  }
			  
			  
				  if (yav == yavA || yav == yavD)
				  {
					  if((xav - xpv) == 0)
					  {
						  newder = (yav-ypv)/(1e-5);
					  }
					  else
					  {
						  newder = (yav-ypv)/(xav - xpv);
					  }
					  
					  if (newder== 0) newder = 1e-5;
					   // fprintf (Outt,"\n xav = %lf  xpv = %lf  yav = %lf  ypv = %lf  newder = %lf IPP = %ld Time = %lf \n",xav,xpv,yav,ypv,newder, ipp,Tp->time );
					  
				  }
				  
				  outvalue = yav;
				  outvalue2 = newder;
		  }
		  break;
	  }
  
 
  }
}


void saltmat4::water_content_relhum (long nid,double *in,double *inp,double *ineq, double *out)
//void saltmat4::water_content_relhum (long nid,double *in,double *inp,double *ineq, double out)
{
  double x1,x1p, ypv1;
 double fi,dfdw;
  
	x1 = in[0];
	x1p = inp[0];
	ypv1 =ineq[11];

	sorption_izoterms_values(MatChar[6],nid,x1,x1p,ypv1,fi,dfdw); 
	
	//fprintf (Outt,"\n Time = %lf xav = %lf  xpv = %lf  yav = %lf  ypv = %lf newder - %3.15e NID = %ld \n",Tp->time ,x1,x1p,fi,ypv1,dfdw, nid);
	
	out[10] = fi;
//	out = fi;			
	
/*
	out[0]=-10000.0;
	out[2]=-10000.0;
	out[3]=-10000.0;
	out[4]=-10000.0;
	out[5]=-10000.0;
	out[6]=-10000.0;
	out[7]=-10000.0;
*/
}
double saltmat4::get_rel_hum2 (double w)
{
	double fi, dfdw;
  	sorption_izoterms_values(MatChar[6],0.0,w,0.0,0.0,fi,dfdw); 
	return (fi);
}


double saltmat4::inverse_hystereze_sorption_izoterms (long ipp, double in, double inp, double ineq)
{
	
  //double xav,yav, newder,ypv,yavA,yavD, ypvA, ypvD, derpvA,derpvD, other1,xpvD, xpvA, xavA, xavD, xpv;
	double xav,yav, newder,ypv, derpvA,derpvD, xpvD, xpvA, xavA, xavD, xpv;
	int hyst;
	int matchar = 6;
  
	yav = in; // relativni vlhkost
	ypv = inp;  // obsah vlhkosti
	xpv = ineq;		  
	
//	CorD(6,other,1,0,xav,yavA,yavD,other1);
// nahrada za CORD ... tady vyhledam pro aktualni relativni vlhkost RH pro Ad a Desorpci
	int pradA, pradD, i;
	pradA  = int(MatData[6][0][0]);
	pradD  = int(MatData[6][2][0]);
	if (yav <=0)
	{
		yav = 0;
		xavA = MatData[6][0][1];
		xavD = MatData[6][2][1];
	}
	else
	{
		if (yav > 1)
		{
			xavA = MatData[6][0][pradA];
			xavD = MatData[6][2][pradD];
		}
		else
		{
			i = 1;
			while (yav > MatData[6][1][i]){
				i++;
			}
			xavA = MatData[6][0][i-1] + ((yav - MatData[6][1][i-1])*(MatData[6][0][i] - MatData[6][0][i-1]))/(MatData[6][1][i] - MatData[6][1][i-1]); 
			i = 1;
			while (yav > MatData[6][3][i]){
				i++;
			}
			xavD = MatData[6][2][i-1] + ((yav - MatData[6][3][i-1])*(MatData[6][2][i] - MatData[6][2][i-1]))/(MatData[6][3][i] - MatData[6][3][i-1]);
		}
	}

	pradA  = int(MatData[6][0][0]);
	pradD  = int(MatData[6][2][0]);
	if (ypv <=0)
	{
		ypv = 0;
		xpvA = MatData[6][0][1];
		xpvD = MatData[6][2][1];
	}
	else
	{
		if (ypv > 1)
		{
			xpvA = MatData[6][0][pradA];
			xpvD = MatData[6][2][pradD];
		}
		else
		{
			i = 1;
			while (ypv > MatData[6][1][i]){
				i++;
			}
			xpvA = MatData[6][0][i-1] + ((ypv - MatData[6][1][i-1])*(MatData[6][0][i] - MatData[6][0][i-1]))/(MatData[6][1][i] - MatData[6][1][i-1]); 
			i = 1;
			while (ypv > MatData[6][3][i]){
				i++;
			}
			xpvD = MatData[6][2][i-1] + ((ypv - MatData[6][3][i-1])*(MatData[6][2][i] - MatData[6][2][i-1]))/(MatData[6][3][i] - MatData[6][3][i-1]);
		}
	}
	derpvA = derivation_dy_dx(6,xpvA,0,1);
	derpvD = derivation_dy_dx(6,xpvD,2,3);
	
	if(yav > ypv)
	{
		hyst = 0;
	}
	else{
		hyst = 1;
	}
	
	switch(hyst)
	{
				case 0:{			// zvysovani promene....
					if(fabs(xpv-xpvA) < 1e-8)                    
					{
						xav = xavA;
					}
					else
					{
						newder = (k1[matchar]*derpvD*(xpv-xpvA)*(xpv-xpvA)+k2[matchar]*derpvA*(xpv -xpvD)*(xpv - xpvD))/((xpvD-xpvA)*(xpvD-xpvA));
						xav = (yav-ypv)*1/newder*k3[matchar]  + xpv;
						if(xav > xavA) xav = xavA;
						if(xav < xavD) xav = xavD;

					}
					break;
					   }
				case 1:{			// klesani zakl promene
					if(fabs(xpv-xpvD) < 1e-8) 
					{
						xav = xavD;
					}
					else {
						newder  = (k4[matchar]*derpvD*(xpv-xpvA)*(xpv-xpvA)+k5[matchar]*derpvA*(xpv -xpvD)*(xpv - xpvD))/((xpvD-xpvA)*(xpvD-xpvA));
						xav = (yav-ypv)*1/newder*k6[matchar] + xpv;
						if(xav > xavA) xav = xavA;
						if(xav < xavD) xav = xavD;

					}
					break;
					   }
	}
	
/*	if (yav == yavA || yav == yavD)
	{
		if ((xav-xpv)==0)
		{
			newder = (yav-ypv)/(1e-5);
		}
		else
		{
			newder = (yav-ypv)/(xav - xpv);
		}
		//  if (newder== 0) newder = 1e-5;
		//fprintf (Outt,"\n xav = %lf  xpv = %lf  yav = %lf  ypv = %lf newder - %lf  IPP = %ld Time = %lf \n",xav,xpv,yav,ypv,newder, ipp,Tp->time );
		
	}
	// if (fabs(yav-ypv)<1e-8) newder = 1e-5;*/

	/*if (Tp->time > 0 && Tp->time < 1*365*86400)
	{
		if(ipp ==4)
		{
			fprintf(Outt2, "%le  %le  %le  %le  %le\n",  Tp->time, xav, yav, xpv, ypv);// kapa a decko
		}
	}*/

return(xav);
	
}


void saltmat4::aux_values_elements (long ipp,double *in,double *inp, double *ineq,double *out)
{
  double x1,x1p,ypv8;
 double fi,dfdw;
  
	x1 = in[0];
  	x1p = inp[0];

	ypv8 =ineq[8];
  
	sorption_izoterms_values(MatChar[6],ipp,x1,x1p,ypv8,fi,dfdw); 
	out[8] = fi;
	out[9] = x1;

/*	if (Tp->time > 0 && Tp->time < 1*365*86400)
	{
		if(ipp ==15)
		{
			fprintf(Outt2, "%le  %le  %le  %le  %le\n",  Tp->time, x1, fi, x1p, ypv8);// kapa a decko
		}
		if(ipp ==0)
		{
			fprintf(Outt1, "%le  %le  %le  %le  %le\n",  Tp->time, x1, fi, x1p, ypv8);// kapa a decko
		}
	}*/
	
}

void saltmat4::read_Sourcet(XFILE *in)
{
double x, y;
long pocet_radku;
	
	xfscanf(in, "%ld",&pocet_radku);
    source = new double *[pocet_radku+1];
	  for( int j = 0; j< pocet_radku+1; j++)
	  {
		  source [j] = new double [2];
	  }

source[0][1] = pocet_radku; 	
	 
 for(int i=1; i< pocet_radku+1; i++)
	{
		xfscanf(in, "%lf %lf",&x, &y);
		source[i][0] = x;
		source[i][1] = y;
	}
source[0][0] = x; 

}

double saltmat4::getval_source (double t)
{
double y;

while(t>=31536000)
	{
	t = t-31536000;
	}	

int i=1;

if (source[i][0] > t)
	{
		return(0.0);
	}


if(source[0][0] > t)
	{
	while (source[i][0] <= t){
		i= i+1;
		}
	   
		if (source[i-1][0]+3600 < source[i][0])
			{
			y = source[i-1][1] +((0.0 - source[i-1][1])/((source[i-1][0]+3600) - source[i-1][0])*(t - source[i-1][0]));
	
			}
		else{
			y = source[i-1][1] +((source[i][1] - source[i-1][1])/(source[i][0] - source[i-1][0])*(t - source[i-1][0]));
			}
}
else{
		i = source[0][1];
		y = source[i][1] +((0.0 - source[i][1])/((source[i][0]+3600) - source[i][0])*(t - source[i][0]));
}
		


if (y<0.0) return(0.0);

return(y/3600);
	
}